-- Generated by ebean unknown at 2021-08-22T03:36:08.393487Z
-- init script create procs
-- Initial script to create stored procedures etc for sqlserver platform

-- create table-value-parameters
if not exists (select name  from sys.types where name = 'ebean_bigint_tvp') create type ebean_bigint_tvp as table (c1 bigint)
GO
if not exists (select name  from sys.types where name = 'ebean_float_tvp') create type ebean_float_tvp as table (c1 float)
GO
if not exists (select name  from sys.types where name = 'ebean_bit_tvp') create type ebean_bit_tvp as table (c1 bit)
GO
if not exists (select name  from sys.types where name = 'ebean_date_tvp') create type ebean_date_tvp as table (c1 date)
GO
if not exists (select name  from sys.types where name = 'ebean_time_tvp') create type ebean_time_tvp as table (c1 time)
GO
if not exists (select name  from sys.types where name = 'ebean_uniqueidentifier_tvp') create type ebean_uniqueidentifier_tvp as table (c1 uniqueidentifier)
GO
if not exists (select name  from sys.types where name = 'ebean_nvarchar_tvp') create type ebean_nvarchar_tvp as table (c1 nvarchar(max))
GO

--
-- PROCEDURE: usp_ebean_drop_indices TABLE, COLUMN
-- deletes all indices referring to TABLE.COLUMN
--
CREATE OR ALTER PROCEDURE usp_ebean_drop_indices @tableName nvarchar(255), @columnName nvarchar(255)
AS SET NOCOUNT ON
declare @sql nvarchar(1000)
declare @indexName nvarchar(255)
BEGIN
  DECLARE index_cursor CURSOR FOR SELECT i.name from sys.indexes i
    join sys.index_columns ic on ic.object_id = i.object_id and ic.index_id = i.index_id
    join sys.columns c on c.object_id = ic.object_id and c.column_id = ic.column_id
    where i.object_id = OBJECT_ID(@tableName) AND c.name = @columnName;
  OPEN index_cursor
  FETCH NEXT FROM index_cursor INTO @indexName
  WHILE @@FETCH_STATUS = 0
    BEGIN
      set @sql = 'drop index ' + @indexName + ' on ' + @tableName;
      EXECUTE(@sql);

      FETCH NEXT FROM index_cursor INTO @indexName
    END;
  CLOSE index_cursor;
  DEALLOCATE index_cursor;
END
GO

--
-- PROCEDURE: usp_ebean_drop_default_constraint TABLE, COLUMN
-- deletes the default constraint, which has a random name
--
CREATE OR ALTER PROCEDURE usp_ebean_drop_default_constraint @tableName nvarchar(255), @columnName nvarchar(255)
AS SET NOCOUNT ON
declare @tmp nvarchar(1000)
BEGIN
  select @tmp = t1.name from sys.default_constraints t1
    join sys.columns t2 on t1.object_id = t2.default_object_id
    where t1.parent_object_id = OBJECT_ID(@tableName) and t2.name = @columnName;

  if @tmp is not null EXEC('alter table ' + @tableName +' drop constraint ' + @tmp);
END
GO

--
-- PROCEDURE: usp_ebean_drop_constraints TABLE, COLUMN
-- deletes constraints and foreign keys refering to TABLE.COLUMN
--
CREATE OR ALTER PROCEDURE usp_ebean_drop_constraints @tableName nvarchar(255), @columnName nvarchar(255)
AS SET NOCOUNT ON
declare @sql nvarchar(1000)
declare @constraintName nvarchar(255)
BEGIN
  DECLARE name_cursor CURSOR FOR
  SELECT cc.name from sys.check_constraints cc
    join sys.columns c on c.object_id = cc.parent_object_id and c.column_id = cc.parent_column_id
    where parent_object_id = OBJECT_ID(@tableName) AND c.name = @columnName
  UNION SELECT fk.name from sys.foreign_keys fk
    join sys.foreign_key_columns fkc on fkc.constraint_object_id = fk.object_id
      and  fkc.parent_object_id = fk.parent_object_id
    join sys.columns c on c.object_id = fkc.parent_object_id and c.column_id = fkc.parent_column_id
    where fkc.parent_object_id = OBJECT_ID(@tableName) AND c.name = @columnName;

  OPEN name_cursor
  FETCH NEXT FROM name_cursor INTO @constraintName
  WHILE @@FETCH_STATUS = 0
    BEGIN
      set @sql = 'alter table ' + @tableName + ' drop constraint ' + @constraintName;
      EXECUTE(@sql);

      FETCH NEXT FROM name_cursor INTO @constraintName
    END;
  CLOSE name_cursor;
  DEALLOCATE name_cursor;
END
GO

--
-- PROCEDURE: usp_ebean_drop_column TABLE, COLUMN
-- deletes the column annd ensures that all indices and constraints are dropped first
--
CREATE OR ALTER PROCEDURE usp_ebean_drop_column @tableName nvarchar(255), @columnName nvarchar(255)
AS SET NOCOUNT ON
declare @sql nvarchar(1000)
BEGIN
  EXEC usp_ebean_drop_indices @tableName, @columnName;
  EXEC usp_ebean_drop_default_constraint @tableName, @columnName;
  EXEC usp_ebean_drop_constraints @tableName, @columnName;

  set @sql = 'alter table ' + @tableName + ' drop column ' + @columnName;
  EXECUTE(@sql);
END
GO
create table asimple_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_asimple_bean primary key (id)
);
create sequence asimple_bean_seq as bigint start with 1 increment by 50;

create table bar (
  bar_type                      nvarchar(31) not null,
  bar_id                        integer not null,
  foo_id                        integer not null,
  version                       integer not null,
  constraint pk_bar primary key (bar_id)
);
create sequence bar_seq as bigint start with 1 increment by 50;

create table block (
  case_type                     integer not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  notes                         nvarchar(255),
  constraint pk_block primary key (id)
);
create sequence block_seq as bigint start with 1 increment by 50;

create table oto_account (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_account primary key (id)
);
create sequence oto_account_seq as bigint start with 1 increment by 50;

create table acl (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_acl primary key (id)
);
create sequence acl_seq as bigint start with 1 increment by 50;

create table acl_container_relation (
  id                            numeric(19) not null,
  container_id                  numeric(19) not null,
  acl_entry_id                  numeric(19) not null,
  constraint pk_acl_container_relation primary key (id)
);
create sequence acl_container_relation_seq as bigint start with 1 increment by 50;

create table addr (
  id                            numeric(19) not null,
  employee_id                   numeric(19),
  name                          nvarchar(255),
  address_line1                 nvarchar(255),
  address_line2                 nvarchar(255),
  city                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_addr primary key (id)
);
create sequence addr_seq as bigint start with 1 increment by 50;

create table address (
  oid                           numeric(19) not null,
  street                        nvarchar(255),
  version                       integer not null,
  constraint pk_address primary key (oid)
);
create sequence address_seq as bigint start with 1 increment by 50;

create table o_address (
  id                            integer not null,
  line_1                        nvarchar(100),
  line_2                        nvarchar(100),
  city                          nvarchar(100),
  cretime                       datetime2,
  country_code                  nvarchar(2),
  updtime                       datetime2 not null,
  constraint pk_o_address primary key (id)
);
create sequence o_address_seq as bigint start with 1 increment by 50;

create table album (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  cover_id                      numeric(19),
  deleted                       integer default 0 not null,
  created_at                    datetime2 not null,
  last_update                   datetime2 not null,
  constraint pk_album primary key (id)
);
create unique nonclustered index uq_album_cover_id on album(cover_id) where cover_id is not null;
create sequence album_seq as bigint start with 1 increment by 50;

create table animal (
  species                       nvarchar(255) not null,
  id                            numeric(19) not null,
  shelter_id                    numeric(19),
  version                       numeric(19) not null,
  name                          nvarchar(255),
  registration_number           nvarchar(255),
  date_of_birth                 date,
  dog_size                      nvarchar(255),
  constraint pk_animal primary key (id)
);
create sequence animal_seq as bigint start with 1 increment by 50;

create table animal_shelter (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_animal_shelter primary key (id)
);
create sequence animal_shelter_seq as bigint start with 1 increment by 50;

create table app_config (
  id                            integer not null,
  constraint pk_app_config primary key (id)
);
create sequence app_config_seq as bigint start with 1 increment by 50;

create table app_config_control (
  id                            integer not null,
  name                          nvarchar(255),
  config_id                     integer,
  constraint pk_app_config_control primary key (id)
);
create sequence app_config_control_seq as bigint start with 1 increment by 50;

create table article (
  id                            integer not null,
  name                          nvarchar(255),
  author                        nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_article primary key (id)
);
create sequence article_seq as bigint start with 1 increment by 50;

create table attribute (
  option_type                   integer not null,
  id                            integer not null,
  attribute_holder_id           integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_attribute primary key (id)
);
create sequence attribute_seq as bigint start with 1 increment by 50;

create table attribute_holder (
  id                            integer not null,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_attribute_holder primary key (id)
);
create sequence attribute_holder_seq as bigint start with 1 increment by 50;

create table audit_log (
  id                            numeric(19) not null,
  description                   nvarchar(255),
  modified_description          nvarchar(255),
  constraint pk_audit_log primary key (id)
);
create sequence audit_log_seq as bigint start with 1000 increment by 50 cache 50;

create table bbookmark (
  id                            integer not null,
  bookmark_reference            nvarchar(255),
  user_id                       integer,
  constraint pk_bbookmark primary key (id)
);
create sequence bbookmark_seq as bigint start with 1 increment by 50;

create table bbookmark_org (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_bbookmark_org primary key (id)
);
create sequence bbookmark_org_seq as bigint start with 1 increment by 50;

create table bbookmark_user (
  id                            integer not null,
  name                          nvarchar(255),
  password                      nvarchar(255),
  email_address                 nvarchar(255),
  country                       nvarchar(255),
  org_id                        integer,
  constraint pk_bbookmark_user primary key (id)
);
create sequence bbookmark_user_seq as bigint start with 1 increment by 50;

create table bsimple_with_gen (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_bsimple_with_gen primary key (id)
);
create sequence bsimple_with_gen_seq as bigint start with 1 increment by 50;

create table bsite (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_bsite primary key (id)
);

create table bsite_user_a (
  site_id                       uniqueidentifier not null,
  user_id                       uniqueidentifier not null,
  access_level                  integer,
  version                       numeric(19) not null,
  constraint ck_bsite_user_a_access_level check ( access_level in (0,1,2)),
  constraint pk_bsite_user_a primary key (site_id,user_id)
);

create table bsite_user_b (
  site                          uniqueidentifier not null,
  usr                           uniqueidentifier not null,
  access_level                  integer,
  constraint ck_bsite_user_b_access_level check ( access_level in (0,1,2)),
  constraint pk_bsite_user_b primary key (site,usr)
);

create table bsite_user_c (
  site_uid                      uniqueidentifier not null,
  user_uid                      uniqueidentifier not null,
  access_level                  integer,
  constraint ck_bsite_user_c_access_level check ( access_level in (0,1,2)),
  constraint pk_bsite_user_c primary key (site_uid,user_uid)
);

create table bsite_user_d (
  site_id                       uniqueidentifier not null,
  user_id                       uniqueidentifier not null,
  access_level                  integer,
  version                       numeric(19) not null,
  constraint ck_bsite_user_d_access_level check ( access_level in (0,1,2)),
  constraint pk_bsite_user_d primary key (site_id,user_id)
);

create table bsite_user_dmany (
  site_id                       uniqueidentifier not null,
  user_id                       uniqueidentifier not null,
  name                          nvarchar(255) not null,
  many                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_bsite_user_dmany primary key (site_id,user_id,name)
);

create table bsite_user_e (
  site_id                       uniqueidentifier not null,
  user_id                       uniqueidentifier not null,
  access_level                  integer,
  constraint ck_bsite_user_e_access_level check ( access_level in (0,1,2))
);

create table buser (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_buser primary key (id)
);

create table bwith_qident (
  id                            integer not null,
  [Name]                        nvarchar(191),
  [CODE]                        nvarchar(255),
  last_updated                  datetime2 not null,
  constraint pk_bwith_qident primary key (id)
);
create unique nonclustered index uq_bwith_qident_name on bwith_qident([Name]) where [Name] is not null;
create sequence bwith_qident_seq as bigint start with 1 increment by 50;

create table basic_draftable_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_basic_draftable_bean primary key (id)
);
create sequence basic_draftable_bean_seq as bigint start with 1 increment by 50;

create table basic_draftable_bean_draft (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_basic_draftable_bean_draft primary key (id)
);
create sequence basic_draftable_bean_draft_seq as bigint start with 1 increment by 50;

create table basic_joda_entity (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  period                        nvarchar(50),
  local_date                    date,
  created                       datetime2 not null,
  updated                       datetime2 not null,
  version                       datetime2 not null,
  constraint pk_basic_joda_entity primary key (id)
);
create sequence basic_joda_entity_seq as bigint start with 1 increment by 50;

create table bean_with_time_zone (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  timezone                      nvarchar(20),
  constraint pk_bean_with_time_zone primary key (id)
);
create sequence bean_with_time_zone_seq as bigint start with 1 increment by 50;

create table drel_booking (
  id                            numeric(19) not null,
  booking_uid                   numeric(19),
  agent_invoice                 numeric(19),
  client_invoice                numeric(19),
  version                       integer not null,
  constraint pk_drel_booking primary key (id)
);
create unique nonclustered index uq_drel_booking_booking_uid on drel_booking(booking_uid) where booking_uid is not null;
create unique nonclustered index uq_drel_booking_agent_invoice on drel_booking(agent_invoice) where agent_invoice is not null;
create unique nonclustered index uq_drel_booking_client_invoice on drel_booking(client_invoice) where client_invoice is not null;
create sequence drel_booking_seq as bigint start with 1 increment by 50;

create table bw_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  flags                         integer not null,
  version                       numeric(19) not null,
  constraint pk_bw_bean primary key (id)
);
create sequence bw_bean_seq as bigint start with 1 increment by 50;

create table cepcategory (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_cepcategory primary key (id)
);
create sequence cepcategory_seq as bigint start with 1 increment by 50;

create table cepproduct (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_cepproduct primary key (id)
);
create sequence cepproduct_seq as bigint start with 1 increment by 50;

create table cepproduct_category (
  customer_id                   numeric(19) not null,
  address_id                    numeric(19) not null,
  category_id                   numeric(19) not null,
  product_id                    numeric(19) not null,
  priority                      integer
);

create table ciaddress (
  id                            numeric(19) not null,
  street_id                     numeric(19),
  constraint pk_ciaddress primary key (id)
);
create sequence ciaddress_seq as bigint start with 1 increment by 50;

create table cicustomer_parent (
  dtype                         integer not null,
  id                            numeric(19) not null,
  address_id                    numeric(19),
  notes                         nvarchar(255),
  constraint pk_cicustomer_parent primary key (id)
);
create sequence cicustomer_parent_seq as bigint start with 1 increment by 50;

create table cistreet_parent (
  dtype                         integer not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  num                           nvarchar(255),
  constraint pk_cistreet_parent primary key (id)
);
create sequence cistreet_parent_seq as bigint start with 1 increment by 50;

create table cinh_ref (
  id                            integer not null,
  ref_id                        integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_cinh_ref primary key (id)
);
create sequence cinh_ref_seq as bigint start with 1 increment by 50;

create table cinh_root (
  dtype                         nvarchar(3) not null,
  id                            integer not null,
  license_number                nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  driver                        nvarchar(255),
  notes                         nvarchar(255),
  action                        nvarchar(255),
  constraint pk_cinh_root primary key (id)
);
create sequence cinh_root_seq as bigint start with 1 increment by 50;

create table ckey_assoc (
  id                            integer not null,
  assoc_one                     nvarchar(255),
  constraint pk_ckey_assoc primary key (id)
);
create sequence ckey_assoc_seq as bigint start with 1 increment by 50;

create table ckey_detail (
  id                            integer not null,
  something                     nvarchar(255),
  one_key                       integer,
  two_key                       nvarchar(127),
  constraint pk_ckey_detail primary key (id)
);
create sequence ckey_detail_seq as bigint start with 1 increment by 50;

create table ckey_parent (
  one_key                       integer not null,
  two_key                       nvarchar(127) not null,
  name                          nvarchar(255),
  assoc_id                      integer,
  version                       integer not null,
  constraint pk_ckey_parent primary key (one_key,two_key)
);

create table coone (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_coone primary key (id)
);
create sequence coone_seq as bigint start with 1 increment by 50;

create table coone_many (
  id                            numeric(19) not null,
  coone_id                      numeric(19) not null,
  name                          nvarchar(255),
  deleted                       integer default 0 not null,
  constraint pk_coone_many primary key (id)
);
create sequence coone_many_seq as bigint start with 1 increment by 50;

create table coroot (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  one_id                        numeric(19),
  constraint pk_coroot primary key (id)
);
create unique nonclustered index uq_coroot_one_id on coroot(one_id) where one_id is not null;
create sequence coroot_seq as bigint start with 1 increment by 50;

create table calcd_data (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_calcd_data primary key (id)
);
create sequence calcd_data_seq as bigint start with 1 increment by 50;

create table calcd_input (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_calcd_input primary key (id)
);
create sequence calcd_input_seq as bigint start with 1 increment by 50;

create table calculation_result (
  id                            integer not null,
  charge                        float(32) not null,
  product_configuration_id      integer,
  group_configuration_id        integer,
  constraint pk_calculation_result primary key (id)
);
create sequence calculation_result_seq as bigint start with 1 increment by 50;

create table cao_bean (
  x_cust_id                     integer not null,
  x_type_id                     integer not null,
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_cao_bean primary key (x_cust_id,x_type_id)
);

create table sp_car_car (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_sp_car_car primary key (id)
);
create sequence sp_car_car_seq as bigint start with 1 increment by 50;

create table sp_car_car_wheels (
  car                           numeric(19) not null,
  wheel                         numeric(19) not null,
  constraint pk_sp_car_car_wheels primary key (car,wheel)
);

create table sp_car_car_doors (
  car                           numeric(19) not null,
  door                          numeric(19) not null,
  constraint pk_sp_car_car_doors primary key (car,door)
);

create table sa_car (
  id                            numeric(19) not null,
  brand                         nvarchar(255),
  sold                          integer not null,
  version                       integer not null,
  constraint pk_sa_car primary key (id)
);
create sequence sa_car_seq as bigint start with 1 increment by 50;

create table car_accessory (
  id                            integer not null,
  name                          nvarchar(255),
  fuse_id                       numeric(19) not null,
  car_id                        integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_car_accessory primary key (id)
);
create sequence car_accessory_seq as bigint start with 1 increment by 50;

create table car_fuse (
  id                            numeric(19) not null,
  location_code                 nvarchar(255),
  constraint pk_car_fuse primary key (id)
);
create sequence car_fuse_seq as bigint start with 1 increment by 50;

create table category (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  surveyobjectid                numeric(19),
  sequence_number               integer not null,
  constraint pk_category primary key (id)
);
create sequence category_seq as bigint start with 1 increment by 50;

create table e_save_test_d (
  id                            numeric(19) not null,
  parent_id                     numeric(19),
  test_property                 integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_e_save_test_d primary key (id)
);
create unique nonclustered index uq_e_save_test_d_parent_id on e_save_test_d(parent_id) where parent_id is not null;
create sequence e_save_test_d_seq as bigint start with 1 increment by 50;

create table child_person (
  identifier                    integer not null,
  name                          nvarchar(255),
  age                           integer,
  some_bean_id                  integer,
  parent_identifier             integer,
  family_name                   nvarchar(255),
  address                       nvarchar(255),
  constraint pk_child_person primary key (identifier)
);
create sequence child_person_seq as bigint start with 1 increment by 50;

create table child_with_cache (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  address                       nvarchar(255),
  constraint pk_child_with_cache primary key (id)
);
create sequence child_with_cache_seq as bigint start with 1 increment by 50;

create table cke_client (
  cod_cpny                      integer not null,
  cod_client                    nvarchar(100) not null,
  username                      nvarchar(100) not null,
  notes                         nvarchar(255),
  constraint pk_cke_client primary key (cod_cpny,cod_client)
);

create table cke_user (
  username                      nvarchar(100) not null,
  cod_cpny                      integer not null,
  name                          nvarchar(255),
  constraint pk_cke_user primary key (username,cod_cpny)
);

create table class_super (
  dtype                         nvarchar(31) not null,
  sid                           numeric(19) not null,
  constraint pk_class_super primary key (sid)
);
create sequence class_super_seq as bigint start with 1 increment by 50;

create table class_super_monkey (
  class_super_sid               numeric(19) not null,
  monkey_mid                    numeric(19) not null,
  constraint uq_class_super_monkey_mid unique (monkey_mid),
  constraint pk_class_super_monkey primary key (class_super_sid,monkey_mid)
);

create table configuration (
  type                          nvarchar(21) not null,
  id                            integer not null,
  name                          nvarchar(255),
  configurations_id             integer,
  group_name                    nvarchar(255),
  product_name                  nvarchar(255),
  constraint pk_configuration primary key (id)
);
create sequence configuration_seq as bigint start with 1 increment by 50;

create table configurations (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_configurations primary key (id)
);
create sequence configurations_seq as bigint start with 1 increment by 50;

create table contact (
  id                            integer not null,
  first_name                    nvarchar(127),
  last_name                     nvarchar(127),
  phone                         nvarchar(255),
  mobile                        nvarchar(255),
  email                         nvarchar(255),
  is_member                     integer default 0 not null,
  customer_id                   integer not null,
  group_id                      integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  constraint pk_contact primary key (id)
);
create sequence contact_seq as bigint start with 1 increment by 50;

create table contact_group (
  id                            integer not null,
  name                          nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_contact_group primary key (id)
);
create sequence contact_group_seq as bigint start with 1 increment by 50;

create table contact_note (
  id                            integer not null,
  contact_id                    integer,
  title                         nvarchar(255),
  note                          nvarchar(max),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_contact_note primary key (id)
);
create sequence contact_note_seq as bigint start with 1 increment by 50;

create table contract (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_contract primary key (id)
);
create sequence contract_seq as bigint start with 1 increment by 50;

create table contract_costs (
  id                            numeric(19) not null,
  status                        nvarchar(255),
  position_id                   numeric(19) not null,
  constraint pk_contract_costs primary key (id)
);
create sequence contract_costs_seq as bigint start with 1 increment by 50;

create table c_conversation (
  id                            numeric(19) not null,
  title                         nvarchar(255),
  isopen                        integer default 0 not null,
  group_id                      numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_c_conversation primary key (id)
);
create sequence c_conversation_seq as bigint start with 1 increment by 50;

create table o_country (
  code                          nvarchar(2) not null,
  name                          nvarchar(60),
  constraint pk_o_country primary key (code)
);

create table cover (
  id                            numeric(19) not null,
  s3_url                        nvarchar(255),
  deleted                       integer default 0 not null,
  constraint pk_cover primary key (id)
);
create sequence cover_seq as bigint start with 1 increment by 50;

create table o_customer (
  id                            integer not null,
  status                        nvarchar(1),
  name                          nvarchar(40) not null,
  smallnote                     nvarchar(100),
  anniversary                   date,
  billing_address_id            integer,
  shipping_address_id           integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint ck_o_customer_status check ( status in ('N','A','I')),
  constraint pk_o_customer primary key (id)
);
create sequence o_customer_seq as bigint start with 1 increment by 50;

create table dcredit (
  id                            numeric(19) not null,
  credit                        nvarchar(255),
  constraint pk_dcredit primary key (id)
);
create sequence dcredit_seq as bigint start with 1 increment by 50;

create table dcredit_drol (
  dcredit_id                    numeric(19) not null,
  drol_id                       numeric(19) not null,
  constraint pk_dcredit_drol primary key (dcredit_id,drol_id)
);

create table dexh_entity (
  oid                           numeric(19) not null,
  exhange                       nvarchar(255),
  an_enum_type                  nvarchar(255),
  last_updated                  datetime2 not null,
  constraint pk_dexh_entity primary key (oid)
);
create sequence dexh_entity_seq as bigint start with 1 increment by 50;

create table dint_parent (
  type                          integer not null,
  id                            numeric(19) not null,
  val                           integer,
  more                          nvarchar(255),
  constraint pk_dint_parent primary key (id)
);
create sequence dint_parent_seq as bigint start with 1 increment by 50;

create table dmachine (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  organisation_id               numeric(19),
  version                       numeric(19) not null,
  constraint pk_dmachine primary key (id)
);
create sequence dmachine_seq as bigint start with 1 increment by 50;

create table d_machine_aux_use (
  id                            numeric(19) not null,
  machine_id                    numeric(19) not null,
  name                          nvarchar(255),
  edate                         date,
  use_secs                      numeric(19) not null,
  fuel                          numeric(16,3),
  version                       numeric(19) not null,
  constraint pk_d_machine_aux_use primary key (id)
);
create sequence d_machine_aux_use_seq as bigint start with 1 increment by 50;

create table d_machine_stats (
  id                            numeric(19) not null,
  machine_id                    numeric(19) not null,
  edate                         date,
  total_kms                     numeric(19) not null,
  hours                         numeric(19) not null,
  rate                          numeric(16,3),
  cost                          numeric(16,3),
  version                       numeric(19) not null,
  constraint pk_d_machine_stats primary key (id)
);
create sequence d_machine_stats_seq as bigint start with 1 increment by 50;

create table d_machine_use (
  id                            numeric(19) not null,
  machine_id                    numeric(19) not null,
  edate                         date,
  distance_kms                  numeric(19) not null,
  time_secs                     numeric(19) not null,
  fuel                          numeric(9,3),
  version                       numeric(19) not null,
  constraint pk_d_machine_use primary key (id)
);
create sequence d_machine_use_seq as bigint start with 1 increment by 50;

create table dorg (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_dorg primary key (id)
);
create sequence dorg_seq as bigint start with 1 increment by 50;

create table dperson (
  id                            numeric(19) not null,
  first_name                    nvarchar(255),
  last_name                     nvarchar(255),
  salary                        numeric(16,3),
  constraint pk_dperson primary key (id)
);
create sequence dperson_seq as bigint start with 1 increment by 50;

create table drol (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_drol primary key (id)
);
create sequence drol_seq as bigint start with 1 increment by 50;

create table drot (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_drot primary key (id)
);
create sequence drot_seq as bigint start with 1 increment by 50;

create table drot_drol (
  drot_id                       numeric(19) not null,
  drol_id                       numeric(19) not null,
  constraint pk_drot_drol primary key (drot_id,drol_id)
);

create table rawinherit_data (
  id                            numeric(19) not null,
  val                           integer,
  constraint pk_rawinherit_data primary key (id)
);
create sequence rawinherit_data_seq as bigint start with 1 increment by 50;

create table data_container (
  id                            uniqueidentifier not null,
  content                       nvarchar(255),
  constraint pk_data_container primary key (id)
);

create table dc_detail (
  id                            numeric(19) not null,
  master_id                     numeric(19),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_dc_detail primary key (id)
);
create sequence dc_detail_seq as bigint start with 1 increment by 50;

create table dc_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_dc_master primary key (id)
);
create sequence dc_master_seq as bigint start with 1 increment by 50;

create table defaults_model (
  id                            integer not null,
  constraint pk_defaults_model primary key (id)
);
create sequence defaults_model_seq as bigint start with 1 increment by 50;

create table defaults_model_draft (
  id                            integer not null,
  constraint pk_defaults_model_draft primary key (id)
);
create sequence defaults_model_draft_seq as bigint start with 1 increment by 50;

create table dfk_cascade (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  one_id                        numeric(19),
  constraint pk_dfk_cascade primary key (id)
);
create sequence dfk_cascade_seq as bigint start with 1 increment by 50;

create table dfk_cascade_one (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_dfk_cascade_one primary key (id)
);
create sequence dfk_cascade_one_seq as bigint start with 1 increment by 50;

create table dfk_none (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  one_id                        numeric(19),
  constraint pk_dfk_none primary key (id)
);
create sequence dfk_none_seq as bigint start with 1 increment by 50;

create table dfk_none_via_join (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  one_id                        numeric(19),
  constraint pk_dfk_none_via_join primary key (id)
);
create sequence dfk_none_via_join_seq as bigint start with 1 increment by 50;

create table dfk_none_via_mto_m (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_dfk_none_via_mto_m primary key (id)
);
create sequence dfk_none_via_mto_m_seq as bigint start with 1 increment by 50;

create table dfk_none_via_mto_m_dfk_one (
  dfk_none_via_mto_m_id         numeric(19) not null,
  dfk_one_id                    numeric(19) not null,
  constraint pk_dfk_none_via_mto_m_dfk_one primary key (dfk_none_via_mto_m_id,dfk_one_id)
);

create table dfk_one (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_dfk_one primary key (id)
);
create sequence dfk_one_seq as bigint start with 1 increment by 50;

create table dfk_set_null (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  one_id                        numeric(19),
  constraint pk_dfk_set_null primary key (id)
);
create sequence dfk_set_null_seq as bigint start with 1 increment by 50;

create table doc (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_doc primary key (id)
);
create sequence doc_seq as bigint start with 1 increment by 50;

create table doc_link (
  doc_id                        numeric(19) not null,
  link_id                       numeric(19) not null,
  constraint pk_doc_link primary key (doc_id,link_id)
);

create table doc_link_draft (
  doc_id                        numeric(19) not null,
  link_id                       numeric(19) not null,
  constraint pk_doc_link_draft primary key (doc_id,link_id)
);

create table doc_draft (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_doc_draft primary key (id)
);
create sequence doc_draft_seq as bigint start with 1 increment by 50;

create table document (
  id                            numeric(19) not null,
  title                         nvarchar(127),
  body                          nvarchar(255),
  organisation_id               numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_document primary key (id)
);
create unique nonclustered index uq_document_title on document(title) where title is not null;
create sequence document_seq as bigint start with 1 increment by 50;

create table document_draft (
  id                            numeric(19) not null,
  title                         nvarchar(127),
  body                          nvarchar(255),
  when_publish                  datetime2,
  organisation_id               numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_document_draft primary key (id)
);
create unique nonclustered index uq_document_draft_title on document_draft(title) where title is not null;
create sequence document_draft_seq as bigint start with 1 increment by 50;

create table document_media (
  id                            numeric(19) not null,
  document_id                   numeric(19),
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_document_media primary key (id)
);
create sequence document_media_seq as bigint start with 1 increment by 50;

create table document_media_draft (
  id                            numeric(19) not null,
  document_id                   numeric(19),
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_document_media_draft primary key (id)
);
create sequence document_media_draft_seq as bigint start with 1 increment by 50;

create table sp_car_door (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_sp_car_door primary key (id)
);
create sequence sp_car_door_seq as bigint start with 1 increment by 50;

create table earray_bean (
  id                            numeric(19) not null,
  foo                           integer,
  name                          nvarchar(255),
  phone_numbers                 varchar(300),
  uids                          varchar(1000) not null,
  other_ids                     varchar(1000),
  doubs                         varchar(1000),
  statuses                      varchar(1000),
  vc_enums                      varchar(1000),
  int_enums                     varchar(1000),
  status2                       varchar(1000),
  version                       numeric(19) not null,
  constraint ck_earray_bean_foo check ( foo in (100,101,102)),
  constraint pk_earray_bean primary key (id)
);
create sequence earray_bean_seq as bigint start with 1 increment by 50;

create table earray_set_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  phone_numbers                 varchar(300),
  uids                          varchar(1000),
  other_ids                     varchar(1000),
  doubs                         varchar(1000),
  status                        varchar(1000),
  version                       numeric(19) not null,
  constraint pk_earray_set_bean primary key (id)
);
create sequence earray_set_bean_seq as bigint start with 1 increment by 50;

create table e_basic (
  id                            integer not null,
  status                        nvarchar(1),
  name                          nvarchar(127),
  description                   nvarchar(255),
  some_date                     datetime2,
  constraint ck_e_basic_status check ( status in ('N','A','I')),
  constraint pk_e_basic primary key (id)
);
create sequence e_basic_seq as bigint start with 1 increment by 50;

create table ebasic_change_log (
  id                            numeric(19) not null,
  name                          nvarchar(20),
  short_description             nvarchar(50),
  long_description              nvarchar(100),
  plain_bean                    nvarchar(500),
  who_created                   nvarchar(255) not null,
  who_modified                  nvarchar(255) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_ebasic_change_log primary key (id)
);
create sequence ebasic_change_log_seq as bigint start with 1 increment by 50;

create table ebasic_clob (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  title                         nvarchar(255),
  description                   nvarchar(max),
  last_update                   datetime2 not null,
  constraint pk_ebasic_clob primary key (id)
);
create sequence ebasic_clob_seq as bigint start with 1 increment by 50;

create table ebasic_clob_fetch_eager (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  title                         nvarchar(255),
  description                   nvarchar(max),
  last_update                   datetime2 not null,
  constraint pk_ebasic_clob_fetch_eager primary key (id)
);
create sequence ebasic_clob_fetch_eager_seq as bigint start with 1 increment by 50;

create table ebasic_clob_no_ver (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  description                   nvarchar(max),
  constraint pk_ebasic_clob_no_ver primary key (id)
);
create sequence ebasic_clob_no_ver_seq as bigint start with 1 increment by 50;

create table e_basicenc (
  id                            integer not null,
  name                          nvarchar(255),
  description                   varbinary(80),
  dob                           varbinary(80),
  status                        varbinary(20),
  last_update                   datetime2,
  constraint pk_e_basicenc primary key (id)
);
create sequence e_basicenc_seq as bigint start with 1 increment by 50;

create table e_basicenc_bin (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  data                          image,
  some_time                     varbinary(255),
  last_update                   datetime2 not null,
  constraint pk_e_basicenc_bin primary key (id)
);
create sequence e_basicenc_bin_seq as bigint start with 1 increment by 50;

create table e_basicenc_client (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  description                   varbinary(80),
  dob                           varbinary(20),
  status                        varbinary(20),
  version                       numeric(19) not null,
  constraint pk_e_basicenc_client primary key (id)
);
create sequence e_basicenc_client_seq as bigint start with 1 increment by 50;

create table e_basicenc_relate (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  other_id                      integer,
  constraint pk_e_basicenc_relate primary key (id)
);
create sequence e_basicenc_relate_seq as bigint start with 1 increment by 50;

create table e_basic_enum_id (
  status                        nvarchar(1) not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  constraint ck_e_basic_enum_id_status check ( status in ('N','A','I')),
  constraint pk_e_basic_enum_id primary key (status)
);

create table e_basic_eni (
  id                            integer not null,
  status                        integer,
  name                          nvarchar(255),
  description                   nvarchar(255),
  some_date                     datetime2,
  constraint ck_e_basic_eni_status check ( status in (1,2,3)),
  constraint pk_e_basic_eni primary key (id)
);
create sequence e_basic_eni_seq as bigint start with 1 increment by 50;

create table ebasic_hstore (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  map                           nvarchar(800),
  version                       numeric(19) not null,
  constraint pk_ebasic_hstore primary key (id)
);
create sequence ebasic_hstore_seq as bigint start with 1 increment by 50;

create table ebasic_json_jackson (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  value_set                     nvarchar(700),
  value_list                    nvarchar(max),
  value_map                     nvarchar(700),
  plain_value                   nvarchar(500),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_jackson primary key (id)
);
create sequence ebasic_json_jackson_seq as bigint start with 1 increment by 50;

create table ebasic_json_jackson2 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  value_set                     nvarchar(700),
  value_list                    nvarchar(max),
  value_map                     nvarchar(700),
  plain_value                   nvarchar(500),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_jackson2 primary key (id)
);
create sequence ebasic_json_jackson2_seq as bigint start with 1 increment by 50;

create table ebasic_json_jackson3 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  plain_value                   nvarchar(500),
  plain_value2                  nvarchar(500),
  plain_value3                  nvarchar(500),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_jackson3 primary key (id)
);
create sequence ebasic_json_jackson3_seq as bigint start with 1 increment by 50;

create table ebasic_json_list (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  beans                         nvarchar(700),
  bean_list                     nvarchar(max),
  bean_map                      nvarchar(700),
  plain_bean                    nvarchar(500),
  flags                         nvarchar(50),
  tags                          nvarchar(100),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_list primary key (id)
);
create sequence ebasic_json_list_seq as bigint start with 1 increment by 50;

create table ebasic_json_map (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map primary key (id)
);
create sequence ebasic_json_map_seq as bigint start with 1 increment by 50;

create table ebasic_json_map_blob (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       image,
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map_blob primary key (id)
);
create sequence ebasic_json_map_blob_seq as bigint start with 1 increment by 50;

create table ebasic_json_map_clob (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map_clob primary key (id)
);
create sequence ebasic_json_map_clob_seq as bigint start with 1 increment by 50;

create table ebasic_json_map_detail (
  id                            numeric(19) not null,
  owner_id                      numeric(19),
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map_detail primary key (id)
);
create sequence ebasic_json_map_detail_seq as bigint start with 1 increment by 50;

create table ebasic_json_map_json_b (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map_json_b primary key (id)
);
create sequence ebasic_json_map_json_b_seq as bigint start with 1 increment by 50;

create table ebasic_json_map_varchar (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(3000),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_map_varchar primary key (id)
);
create sequence ebasic_json_map_varchar_seq as bigint start with 1 increment by 50;

create table ebasic_json_multi (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  plain_value1                  nvarchar(500),
  plain_value2                  nvarchar(500),
  plain_value3                  nvarchar(500),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_multi primary key (id)
);
create sequence ebasic_json_multi_seq as bigint start with 1 increment by 50;

create table ebasic_json_node (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_node primary key (id)
);
create sequence ebasic_json_node_seq as bigint start with 1 increment by 50;

create table ebasic_json_node_blob (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       image,
  version                       numeric(19) not null,
  constraint pk_ebasic_json_node_blob primary key (id)
);
create sequence ebasic_json_node_blob_seq as bigint start with 1 increment by 50;

create table ebasic_json_node_json_b (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(max),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_node_json_b primary key (id)
);
create sequence ebasic_json_node_json_b_seq as bigint start with 1 increment by 50;

create table ebasic_json_node_varchar (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       nvarchar(1000),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_node_varchar primary key (id)
);
create sequence ebasic_json_node_varchar_seq as bigint start with 1 increment by 50;

create table ebasic_json_unmapped (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ebasic_json_unmapped primary key (id)
);
create sequence ebasic_json_unmapped_seq as bigint start with 1 increment by 50;

create table e_basic_log (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_e_basic_log primary key (id)
);
create sequence e_basic_log_seq as bigint start with 1 increment by 50;

create table e_basic_ndc (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_e_basic_ndc primary key (id)
);
create sequence e_basic_ndc_seq as bigint start with 1 increment by 50;

create table ebasic_no_sdchild (
  id                            numeric(19) not null,
  owner_id                      numeric(19) not null,
  child_name                    nvarchar(255),
  amount                        numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_ebasic_no_sdchild primary key (id)
);
create sequence ebasic_no_sdchild_seq as bigint start with 1 increment by 50;

create table ebasic_old_value (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  string_set                    nvarchar(max),
  long_set                      nvarchar(max),
  int_set                       nvarchar(max),
  string_list                   nvarchar(max),
  long_list                     nvarchar(max),
  int_list                      nvarchar(max),
  object_map                    nvarchar(max),
  long_map                      nvarchar(max),
  int_map                       nvarchar(max),
  json_node                     nvarchar(max),
  string_arr                    varchar(1000),
  constraint pk_ebasic_old_value primary key (id)
);
create sequence ebasic_old_value_seq as bigint start with 1 increment by 50;

create table ebasic_plain (
  id                            numeric(19) not null,
  attr                          nvarchar(255),
  plain_bean                    nvarchar(500),
  plain_bean2                   nvarchar(500),
  version                       numeric(19) not null,
  constraint pk_ebasic_plain primary key (id)
);
create sequence ebasic_plain_seq as bigint start with 1 increment by 50;

create table ebasic_sdchild (
  id                            numeric(19) not null,
  owner_id                      numeric(19) not null,
  child_name                    nvarchar(255),
  amount                        numeric(19) not null,
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_ebasic_sdchild primary key (id)
);
create sequence ebasic_sdchild_seq as bigint start with 1 increment by 50;

create table ebasic_soft_delete (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_ebasic_soft_delete primary key (id)
);
create sequence ebasic_soft_delete_seq as bigint start with 1 increment by 50;

create table e_basicver (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  other                         nvarchar(255),
  last_update                   datetime2 not null,
  constraint pk_e_basicver primary key (id)
);
create sequence e_basicver_seq as bigint start with 1 increment by 50;

create table e_basic_withlife (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  other                         nvarchar(255),
  deleted                       integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_e_basic_withlife primary key (id)
);
create sequence e_basic_withlife_seq as bigint start with 1 increment by 50;

create table e_basic_with_ex (
  id                            numeric(19) not null,
  deleted                       integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_e_basic_with_ex primary key (id)
);
create sequence e_basic_with_ex_seq as bigint start with 1 increment by 50;

create table e_basic_withlog (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  deleted                       integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_e_basic_withlog primary key (id)
);
create sequence e_basic_withlog_seq as bigint start with 1 increment by 50;

create table e_basicverucon (
  id                            integer not null,
  name                          nvarchar(127),
  other                         nvarchar(127),
  other_one                     nvarchar(127),
  description                   nvarchar(255),
  last_update                   datetime2 not null,
  constraint pk_e_basicverucon primary key (id)
);
create unique nonclustered index uq_e_basicverucon_name on e_basicverucon(name) where name is not null;
create unique nonclustered index uq_e_basicverucon_other_other_one on e_basicverucon(other,other_one) where other is not null and other_one is not null;
create sequence e_basicverucon_seq as bigint start with 1 increment by 50;

create table ecache_child (
  id                            uniqueidentifier not null,
  name                          nvarchar(100),
  root_id                       uniqueidentifier not null,
  constraint pk_ecache_child primary key (id)
);

create table ecache_root (
  id                            uniqueidentifier not null,
  name                          nvarchar(100),
  constraint pk_ecache_root primary key (id)
);

create table e_col_ab (
  id                            numeric(19) not null,
  column_a                      nvarchar(255),
  column_b                      nvarchar(255),
  constraint pk_e_col_ab primary key (id)
);
create sequence e_col_ab_seq as bigint start with 1 increment by 50;

create table ecustom_id (
  id                            nvarchar(127) not null,
  name                          nvarchar(255),
  constraint pk_ecustom_id primary key (id)
);

create table edefault_prop (
  id                            integer not null,
  e_simple_usertypeid           integer,
  name                          nvarchar(255),
  constraint pk_edefault_prop primary key (id)
);
create unique nonclustered index uq_edefault_prop_e_simple_usertypeid on edefault_prop(e_simple_usertypeid) where e_simple_usertypeid is not null;
create sequence edefault_prop_seq as bigint start with 1 increment by 50;

create table eemb_inner (
  id                            integer not null,
  nome_inner                    nvarchar(255),
  outer_id                      integer,
  update_count                  integer not null,
  constraint pk_eemb_inner primary key (id)
);
create sequence eemb_inner_seq as bigint start with 1 increment by 50;

create table eemb_outer (
  id                            integer not null,
  nome_outer                    nvarchar(255),
  date1                         datetime2,
  date2                         datetime2,
  update_count                  integer not null,
  constraint pk_eemb_outer primary key (id)
);
create sequence eemb_outer_seq as bigint start with 1 increment by 50;

create table efile2_no_fk (
  file_name                     nvarchar(64) not null,
  owner_id                      integer not null,
  constraint pk_efile2_no_fk primary key (file_name)
);

create table efile_no_fk (
  file_name                     nvarchar(64) not null,
  owner_user_id                 integer,
  owner_soft_del_user_id        integer,
  constraint pk_efile_no_fk primary key (file_name)
);

create table efile_no_fk_euser_no_fk (
  efile_no_fk_file_name         nvarchar(64) not null,
  euser_no_fk_user_id           integer not null,
  constraint pk_efile_no_fk_euser_no_fk primary key (efile_no_fk_file_name,euser_no_fk_user_id)
);

create table efile_no_fk_euser_no_fk_soft_del (
  efile_no_fk_file_name         nvarchar(64) not null,
  euser_no_fk_soft_del_user_id  integer not null,
  constraint pk_efile_no_fk_euser_no_fk_soft_del primary key (efile_no_fk_file_name,euser_no_fk_soft_del_user_id)
);

create table egen_props (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  ts_created                    datetime2 not null,
  ts_updated                    datetime2 not null,
  ldt_created                   datetime2 not null,
  ldt_updated                   datetime2 not null,
  odt_created                   datetime2 not null,
  odt_updated                   datetime2 not null,
  zdt_created                   datetime2 not null,
  zdt_updated                   datetime2 not null,
  instant_created               datetime2 not null,
  instant_updated               datetime2 not null,
  long_created                  numeric(19) not null,
  long_updated                  numeric(19) not null,
  constraint pk_egen_props primary key (id)
);
create sequence egen_props_seq as bigint start with 1 increment by 50;

create table eid_uid_bean (
  id                            numeric(19) not null,
  uuid                          uniqueidentifier not null,
  name                          nvarchar(255),
  constraint uq_eid_uid_bean_uuid unique (uuid),
  constraint pk_eid_uid_bean primary key (id)
);
create sequence eid_uid_bean_seq as bigint start with 1 increment by 50;

create table einvoice (
  id                            numeric(19) not null,
  invoice_date                  datetime2,
  state                         integer,
  person_id                     numeric(19),
  ship_street                   nvarchar(255),
  ship_suburb                   nvarchar(255),
  ship_city                     nvarchar(255),
  ship_status                   nvarchar(3),
  ship_jbean                    nvarchar(max),
  ship_jraw                     nvarchar(max),
  bill_street                   nvarchar(255),
  bill_suburb                   nvarchar(255),
  bill_city                     nvarchar(255),
  bill_status                   nvarchar(3),
  bill_jbean                    nvarchar(max),
  bill_jraw                     nvarchar(max),
  version                       numeric(19) not null,
  constraint ck_einvoice_state check ( state in (0,1,2)),
  constraint ck_einvoice_ship_status check ( ship_status in ('ONE','TWO')),
  constraint ck_einvoice_bill_status check ( bill_status in ('ONE','TWO')),
  constraint pk_einvoice primary key (id)
);
create sequence einvoice_seq as bigint start with 1 increment by 50;

create table e_main (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_e_main primary key (id)
);
create sequence e_main_seq as bigint start with 1 increment by 50;

create table enull_collection (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_enull_collection primary key (id)
);
create sequence enull_collection_seq as bigint start with 1 increment by 50;

create table enull_collection_detail (
  id                            integer not null,
  enull_collection_id           integer not null,
  something                     nvarchar(255),
  constraint pk_enull_collection_detail primary key (id)
);
create sequence enull_collection_detail_seq as bigint start with 1 increment by 50;

create table eopt_one_a (
  id                            integer not null,
  name_for_a                    nvarchar(255),
  b_id                          integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_eopt_one_a primary key (id)
);
create sequence eopt_one_a_seq as bigint start with 1 increment by 50;

create table eopt_one_b (
  id                            integer not null,
  name_for_b                    nvarchar(255),
  c_id                          integer not null,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_eopt_one_b primary key (id)
);
create sequence eopt_one_b_seq as bigint start with 1 increment by 50;

create table eopt_one_c (
  id                            integer not null,
  name_for_c                    nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_eopt_one_c primary key (id)
);
create sequence eopt_one_c_seq as bigint start with 1 increment by 50;

create table eper_addr (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  ma_street                     nvarchar(255),
  ma_suburb                     nvarchar(255),
  ma_city                       nvarchar(255),
  ma_country_code               nvarchar(2),
  version                       numeric(19) not null,
  constraint pk_eper_addr primary key (id)
);
create sequence eper_addr_seq as bigint start with 1 increment by 50;

create table eperson (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  notes                         nvarchar(255),
  street                        nvarchar(255),
  suburb                        nvarchar(255),
  addr_city                     nvarchar(255),
  addr_status                   nvarchar(3),
  addr_jbean                    nvarchar(max),
  jraw                          nvarchar(max),
  version                       numeric(19) not null,
  constraint ck_eperson_addr_status check ( addr_status in ('ONE','TWO')),
  constraint pk_eperson primary key (id)
);
create sequence eperson_seq as bigint start with 1 increment by 50;

create table eperson2 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  notes                         nvarchar(255),
  street                        nvarchar(10),
  suburb                        nvarchar(100),
  city                          nvarchar(255),
  status                        nvarchar(3),
  jbean                         nvarchar(max),
  jraw                          nvarchar(max),
  version                       numeric(19) not null,
  constraint ck_eperson2_status check ( status in ('ONE','TWO')),
  constraint pk_eperson2 primary key (id)
);
create sequence eperson2_seq as bigint start with 1 increment by 50;

create table eperson3 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  street                        nvarchar(255),
  suburb                        nvarchar(255),
  city                          nvarchar(255),
  status                        nvarchar(3),
  jbean                         nvarchar(max),
  jraw                          nvarchar(max),
  version                       numeric(19) not null,
  constraint ck_eperson3_status check ( status in ('ONE','TWO')),
  constraint pk_eperson3 primary key (id)
);
create sequence eperson3_seq as bigint start with 1 increment by 50;

create table e_person_online (
  id                            numeric(19) not null,
  email                         nvarchar(127),
  online_status                 integer default 0 not null,
  when_updated                  datetime2 not null,
  constraint pk_e_person_online primary key (id)
);
create unique nonclustered index uq_e_person_online_email on e_person_online(email) where email is not null;
create sequence e_person_online_seq as bigint start with 1 increment by 50;

create table esimple (
  usertypeid                    integer identity(1,1) not null,
  name                          nvarchar(255),
  constraint pk_esimple primary key (usertypeid)
);

create table esoft_del_book (
  id                            numeric(19) not null,
  book_title                    nvarchar(255),
  lend_by_id                    numeric(19),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_book primary key (id)
);
create sequence esoft_del_book_seq as bigint start with 1 increment by 50;

create table esoft_del_book_esoft_del_user (
  esoft_del_book_id             numeric(19) not null,
  esoft_del_user_id             numeric(19) not null,
  constraint pk_esoft_del_book_esoft_del_user primary key (esoft_del_book_id,esoft_del_user_id)
);

create table esoft_del_down (
  id                            numeric(19) not null,
  esoft_del_mid_id              numeric(19) not null,
  down                          nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_down primary key (id)
);
create sequence esoft_del_down_seq as bigint start with 1 increment by 50;

create table esoft_del_mid (
  id                            numeric(19) not null,
  top_id                        numeric(19),
  mid                           nvarchar(255),
  up_id                         numeric(19),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_mid primary key (id)
);
create sequence esoft_del_mid_seq as bigint start with 1 increment by 50;

create table esoft_del_one_a (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  oneb_id                       numeric(19),
  deleted                       integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_esoft_del_one_a primary key (id)
);
create unique nonclustered index uq_esoft_del_one_a_oneb_id on esoft_del_one_a(oneb_id) where oneb_id is not null;
create sequence esoft_del_one_a_seq as bigint start with 1 increment by 50;

create table esoft_del_one_b (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_esoft_del_one_b primary key (id)
);
create sequence esoft_del_one_b_seq as bigint start with 1 increment by 50;

create table esoft_del_role (
  id                            numeric(19) not null,
  role_name                     nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_role primary key (id)
);
create sequence esoft_del_role_seq as bigint start with 1 increment by 50;

create table esoft_del_role_esoft_del_user (
  esoft_del_role_id             numeric(19) not null,
  esoft_del_user_id             numeric(19) not null,
  constraint pk_esoft_del_role_esoft_del_user primary key (esoft_del_role_id,esoft_del_user_id)
);

create table esoft_del_top (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_top primary key (id)
);
create sequence esoft_del_top_seq as bigint start with 1 increment by 50;

create table esoft_del_up (
  id                            numeric(19) not null,
  up                            nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_up primary key (id)
);
create sequence esoft_del_up_seq as bigint start with 1 increment by 50;

create table esoft_del_user (
  id                            numeric(19) not null,
  user_name                     nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_user primary key (id)
);
create sequence esoft_del_user_seq as bigint start with 1 increment by 50;

create table esoft_del_user_esoft_del_role (
  esoft_del_user_id             numeric(19) not null,
  esoft_del_role_id             numeric(19) not null,
  constraint pk_esoft_del_user_esoft_del_role primary key (esoft_del_user_id,esoft_del_role_id)
);

create table esoft_del_x (
  id                            uniqueidentifier not null,
  y_id                          numeric(19),
  organization_id               numeric(19),
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_x primary key (id)
);
create unique nonclustered index uq_esoft_del_x_y_id on esoft_del_x(y_id) where y_id is not null;

create table esoft_del_y (
  id                            numeric(19) not null,
  organization_id               numeric(19),
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_y primary key (id)
);
create sequence esoft_del_y_seq as bigint start with 1 increment by 50;

create table esoft_del_z (
  id                            numeric(19) not null,
  uuid                          uniqueidentifier,
  deleted                       integer default 0 not null,
  constraint pk_esoft_del_z primary key (id)
);
create sequence esoft_del_z_seq as bigint start with 1 increment by 50;

create table esome_convert_type (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  money                         numeric(16,3),
  constraint pk_esome_convert_type primary key (id)
);
create sequence esome_convert_type_seq as bigint start with 1 increment by 50;

create table esome_type (
  id                            integer not null,
  currency                      nvarchar(3),
  locale                        nvarchar(20),
  time_zone                     nvarchar(20),
  constraint pk_esome_type primary key (id)
);
create sequence esome_type_seq as bigint start with 1 increment by 50;

create table etrans_many (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_etrans_many primary key (id)
);
create sequence etrans_many_seq as bigint start with 1 increment by 50;

create table rawinherit_uncle (
  id                            integer not null,
  name                          nvarchar(255),
  parent_id                     numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_rawinherit_uncle primary key (id)
);
create sequence rawinherit_uncle_seq as bigint start with 1 increment by 50;

create table euser_no_fk (
  user_id                       integer not null,
  user_name                     nvarchar(255),
  constraint pk_euser_no_fk primary key (user_id)
);
create sequence euser_no_fk_seq as bigint start with 1 increment by 50;

create table euser_no_fk_soft_del (
  user_id                       integer not null,
  user_name                     nvarchar(255),
  constraint pk_euser_no_fk_soft_del primary key (user_id)
);
create sequence euser_no_fk_soft_del_seq as bigint start with 1 increment by 50;

create table evanilla_collection (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_evanilla_collection primary key (id)
);
create sequence evanilla_collection_seq as bigint start with 1 increment by 50;

create table evanilla_collection_detail (
  id                            integer not null,
  evanilla_collection_id        integer not null,
  something                     nvarchar(255),
  constraint pk_evanilla_collection_detail primary key (id)
);
create sequence evanilla_collection_detail_seq as bigint start with 1 increment by 50;

create table ewho_props (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  who_created                   nvarchar(255) not null,
  who_modified                  nvarchar(255) not null,
  constraint pk_ewho_props primary key (id)
);
create sequence ewho_props_seq as bigint start with 1 increment by 50;

create table e_withinet (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  inet_address                  nvarchar(50),
  inet2                         nvarchar(255),
  cidr                          varchar(50),
  version                       numeric(19) not null,
  constraint pk_e_withinet primary key (id)
);
create sequence e_withinet_seq as bigint start with 1 increment by 50;

create table ec_enum_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ec_enum_person primary key (id)
);
create sequence ec_enum_person_seq as bigint start with 1 increment by 50;

create table ec_enum_person_tags (
  ec_enum_person_id             numeric(19) not null,
  value                         nvarchar(5) not null,
  constraint ck_ec_enum_person_tags_value check ( value in ('RED','BLUE','GREEN'))
);

create table ec_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ec_person primary key (id)
);
create sequence ec_person_seq as bigint start with 1 increment by 50;

create table ec_person_phone (
  owner_id                      numeric(19) not null,
  phone                         nvarchar(255) not null
);

create table ec_top (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  person_id                     numeric(19),
  version                       numeric(19) not null,
  constraint pk_ec_top primary key (id)
);
create sequence ec_top_seq as bigint start with 1 increment by 50;

create table ec_top_ecs_person (
  ec_top_id                     numeric(19) not null,
  ecs_person_id                 numeric(19) not null,
  constraint pk_ec_top_ecs_person primary key (ec_top_id,ecs_person_id)
);

create table ecbl_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecbl_person primary key (id)
);
create sequence ecbl_person_seq as bigint start with 1 increment by 50;

create table ecbl_person_phone_numbers (
  person_id                     numeric(19) not null,
  country_code                  nvarchar(2),
  area                          nvarchar(6),
  phnum                         nvarchar(20)
);

create table ecbm_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecbm_person primary key (id)
);
create sequence ecbm_person_seq as bigint start with 1 increment by 50;

create table ecbm_person_phone_numbers (
  person_id                     numeric(19) not null,
  mkey                          nvarchar(255) not null,
  country_code                  nvarchar(2),
  area                          nvarchar(6),
  phnum                         nvarchar(20)
);

create table ecm_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecm_person primary key (id)
);
create sequence ecm_person_seq as bigint start with 1 increment by 50;

create table ecm_person_phone_numbers (
  ecm_person_id                 numeric(19) not null,
  type                          nvarchar(4) not null,
  phnum                         nvarchar(10) not null
);

create table ecmc_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecmc_person primary key (id)
);
create sequence ecmc_person_seq as bigint start with 1 increment by 50;

create table ecmc_person_phone_numbers (
  ecmc_person_id                numeric(19) not null,
  type                          nvarchar(4) not null,
  value                         nvarchar(max) not null
);

create table ecs_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecs_person primary key (id)
);
create sequence ecs_person_seq as bigint start with 1 increment by 50;

create table ecs_person_phone (
  ecs_person_id                 numeric(19) not null,
  phone                         nvarchar(255) not null
);

create table ecsm_child (
  one_id                        uniqueidentifier not null,
  ecsm_parent_id                numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecsm_child primary key (one_id)
);

create table ecsm_values (
  host_id                       uniqueidentifier not null,
  value                         nvarchar(255) not null
);

create table ecsm_one (
  one_id                        uniqueidentifier not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecsm_one primary key (one_id)
);

create table ecsm_parent (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecsm_parent primary key (id)
);
create sequence ecsm_parent_seq as bigint start with 1 increment by 50;

create table ecsm_two (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ecsm_two primary key (id)
);

create table td_child (
  child_id                      integer not null,
  child_name                    nvarchar(255),
  parent_id                     integer not null,
  constraint pk_td_child primary key (child_id)
);
create sequence td_child_seq as bigint start with 1 increment by 50;

create table td_parent (
  parent_type                   nvarchar(31) not null,
  parent_id                     integer not null,
  parent_name                   nvarchar(255),
  extended_name                 nvarchar(255),
  constraint pk_td_parent primary key (parent_id)
);
create sequence td_parent_seq as bigint start with 1 increment by 50;

create table element_bean (
  id                            numeric(19) not null,
  complex_bean_id               uniqueidentifier not null,
  value                         nvarchar(255) not null,
  constraint pk_element_bean primary key (id)
);
create sequence element_bean_seq as bigint start with 1 increment by 50;

create table empl (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  age                           integer,
  default_address_id            numeric(19),
  constraint pk_empl primary key (id)
);
create sequence empl_seq as bigint start with 1 increment by 50;

create table esd_detail (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  master_id                     numeric(19) not null,
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esd_detail primary key (id)
);
create sequence esd_detail_seq as bigint start with 1 increment by 50;

create table esd_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_esd_master primary key (id)
);
create sequence esd_master_seq as bigint start with 1 increment by 50;

create table feature_desc (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  constraint pk_feature_desc primary key (id)
);
create sequence feature_desc_seq as bigint start with 1 increment by 50;

create table f_first (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_f_first primary key (id)
);
create sequence f_first_seq as bigint start with 1 increment by 50;

create table foo (
  foo_id                        integer not null,
  important_text                nvarchar(255),
  version                       integer not null,
  constraint pk_foo primary key (foo_id)
);
create sequence foo_seq as bigint start with 1 increment by 50;

create table gen_key_identity (
  id                            numeric(19) identity(1,1) not null,
  description                   nvarchar(255),
  constraint pk_gen_key_identity primary key (id)
);

create table gen_key_sequence (
  id                            numeric(19) not null,
  description                   nvarchar(255),
  constraint pk_gen_key_sequence primary key (id)
);
create sequence SEQ_NAME as bigint start with 1 increment by 50;

create table grand_parent_person (
  identifier                    integer not null,
  name                          nvarchar(255),
  age                           integer,
  some_bean_id                  integer,
  family_name                   nvarchar(255),
  address                       nvarchar(255),
  constraint pk_grand_parent_person primary key (identifier)
);
create sequence grand_parent_person_seq as bigint start with 1 increment by 50;

create table survey_group (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  categoryobjectid              numeric(19),
  sequence_number               integer not null,
  constraint pk_survey_group primary key (id)
);
create sequence survey_group_seq as bigint start with 1 increment by 50;

create table c_group (
  id                            numeric(19) not null,
  inactive                      integer default 0 not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_c_group primary key (id)
);
create sequence c_group_seq as bigint start with 1 increment by 50;

create table hembi_bean (
  part                          numeric(19) not null,
  brand                         nvarchar(20) not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_hembi_bean primary key (part,brand)
);

create table he_doc (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_he_doc primary key (id)
);
create sequence he_doc_seq as bigint start with 1 increment by 50;

create table hx_link (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  location                      nvarchar(255),
  comments                      nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  deleted                       integer default 0 not null,
  constraint pk_hx_link primary key (id)
);
create sequence hx_link_seq as bigint start with 1 increment by 50;

create table hx_link_doc (
  hx_link_id                    numeric(19) not null,
  he_doc_id                     numeric(19) not null,
  constraint pk_hx_link_doc primary key (hx_link_id,he_doc_id)
);

create table hi_doc (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_hi_doc primary key (id)
);
create sequence hi_doc_seq as bigint start with 1 increment by 50;

create table hi_link (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  location                      nvarchar(255),
  comments                      nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_hi_link primary key (id)
);
create sequence hi_link_seq as bigint start with 1 increment by 50;

create table hi_link_doc (
  hi_link_id                    numeric(19) not null,
  hi_doc_id                     numeric(19) not null,
  constraint pk_hi_link_doc primary key (hi_link_id,hi_doc_id)
);

create table hi_tone (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  comments                      nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_hi_tone primary key (id)
);
create sequence hi_tone_seq as bigint start with 1 increment by 50;

create table hi_tthree (
  id                            numeric(19) not null,
  hi_ttwo_id                    numeric(19) not null,
  three                         nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_hi_tthree primary key (id)
);
create sequence hi_tthree_seq as bigint start with 1 increment by 50;

create table hi_ttwo (
  id                            numeric(19) not null,
  hi_tone_id                    numeric(19) not null,
  two                           nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_hi_ttwo primary key (id)
);
create sequence hi_ttwo_seq as bigint start with 1 increment by 50;

create table hsd_setting (
  id                            numeric(19) not null,
  code                          nvarchar(255),
  content                       nvarchar(255),
  user_id                       numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  deleted                       integer default 0 not null,
  constraint pk_hsd_setting primary key (id)
);
create unique nonclustered index uq_hsd_setting_user_id on hsd_setting(user_id) where user_id is not null;
create sequence hsd_setting_seq as bigint start with 1 increment by 50;

create table hsd_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  deleted                       integer default 0 not null,
  constraint pk_hsd_user primary key (id)
);
create sequence hsd_user_seq as bigint start with 1 increment by 50;

create table iaf_segment (
  ptype                         nvarchar(31) not null,
  id                            numeric(19) not null,
  segment_id_zat                numeric(19) not null,
  status_id                     numeric(19) not null,
  constraint pk_iaf_segment primary key (id)
);
create sequence iaf_segment_seq as bigint start with 1 increment by 50;

create table iaf_segment_status (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_iaf_segment_status primary key (id)
);
create sequence iaf_segment_status_seq as bigint start with 1 increment by 50;

create table imrelated (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  owner_id                      numeric(19) not null,
  constraint pk_imrelated primary key (id)
);
create sequence imrelated_seq as bigint start with 1 increment by 50;

create table imroot (
  dtype                         nvarchar(31) not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  title                         nvarchar(255),
  when_title                    datetime2,
  constraint pk_imroot primary key (id)
);
create sequence imroot_seq as bigint start with 1 increment by 50;

create table ixresource (
  dtype                         nvarchar(255),
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_ixresource primary key (id)
);

create table info_company (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_info_company primary key (id)
);
create sequence info_company_seq as bigint start with 1 increment by 50;

create table info_contact (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  company_id                    numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_info_contact primary key (id)
);
create sequence info_contact_seq as bigint start with 1 increment by 50;

create table info_customer (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  company_id                    numeric(19),
  version                       numeric(19) not null,
  constraint pk_info_customer primary key (id)
);
create unique nonclustered index uq_info_customer_company_id on info_customer(company_id) where company_id is not null;
create sequence info_customer_seq as bigint start with 1 increment by 50;

create table inner_report (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  forecast_id                   numeric(19),
  constraint pk_inner_report primary key (id)
);
create unique nonclustered index uq_inner_report_forecast_id on inner_report(forecast_id) where forecast_id is not null;
create sequence inner_report_seq as bigint start with 1 increment by 50;

create table drel_invoice (
  id                            numeric(19) not null,
  booking                       numeric(19),
  version                       integer not null,
  constraint pk_drel_invoice primary key (id)
);
create sequence drel_invoice_seq as bigint start with 1 increment by 50;

create table item (
  customer                      integer not null,
  itemnumber                    nvarchar(127) not null,
  description                   nvarchar(255),
  units                         nvarchar(255),
  type                          integer not null,
  region                        integer not null,
  date_modified                 datetime2,
  date_created                  datetime2,
  modified_by                   nvarchar(255),
  created_by                    nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_item primary key (customer,itemnumber)
);

create table monkey (
  mid                           numeric(19) not null,
  name                          nvarchar(255),
  food_preference               nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_monkey primary key (mid)
);
create sequence monkey_seq as bigint start with 1 increment by 50;

create table mkeygroup (
  pid                           numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mkeygroup primary key (pid)
);
create sequence mkeygroup_seq as bigint start with 1 increment by 50;

create table mkeygroup_monkey (
  mkeygroup_pid                 numeric(19) not null,
  monkey_mid                    numeric(19) not null,
  constraint uq_mkeygroup_monkey_mid unique (monkey_mid),
  constraint pk_mkeygroup_monkey primary key (mkeygroup_pid,monkey_mid)
);

create table trainer (
  tid                           numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_trainer primary key (tid)
);
create sequence trainer_seq as bigint start with 1 increment by 50;

create table trainer_monkey (
  trainer_tid                   numeric(19) not null,
  monkey_mid                    numeric(19) not null,
  constraint uq_trainer_monkey_mid unique (monkey_mid),
  constraint pk_trainer_monkey primary key (trainer_tid,monkey_mid)
);

create table troop (
  pid                           numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_troop primary key (pid)
);
create sequence troop_seq as bigint start with 1 increment by 50;

create table troop_monkey (
  troop_pid                     numeric(19) not null,
  monkey_mid                    numeric(19) not null,
  constraint uq_troop_monkey_mid unique (monkey_mid),
  constraint pk_troop_monkey primary key (troop_pid,monkey_mid)
);

create table l2_cldf_reset_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_l2_cldf_reset_bean primary key (id)
);
create sequence l2_cldf_reset_bean_seq as bigint start with 1 increment by 50;

create table l2_cldf_reset_bean_child (
  id                            numeric(19) not null,
  parent_id                     numeric(19),
  constraint pk_l2_cldf_reset_bean_child primary key (id)
);
create sequence l2_cldf_reset_bean_child_seq as bigint start with 1 increment by 50;

create table level1 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_level1 primary key (id)
);
create sequence level1_seq as bigint start with 1 increment by 50;

create table level1_level4 (
  level1_id                     numeric(19) not null,
  level4_id                     numeric(19) not null,
  constraint pk_level1_level4 primary key (level1_id,level4_id)
);

create table level1_level2 (
  level1_id                     numeric(19) not null,
  level2_id                     numeric(19) not null,
  constraint pk_level1_level2 primary key (level1_id,level2_id)
);

create table level2 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_level2 primary key (id)
);
create sequence level2_seq as bigint start with 1 increment by 50;

create table level2_level3 (
  level2_id                     numeric(19) not null,
  level3_id                     numeric(19) not null,
  constraint pk_level2_level3 primary key (level2_id,level3_id)
);

create table level3 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_level3 primary key (id)
);
create sequence level3_seq as bigint start with 1 increment by 50;

create table level4 (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_level4 primary key (id)
);
create sequence level4_seq as bigint start with 1 increment by 50;

create table link (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  location                      nvarchar(255),
  when_publish                  datetime2,
  link_comment                  nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  deleted                       integer default 0 not null,
  constraint pk_link primary key (id)
);
create sequence link_seq as bigint start with 1 increment by 50;

create table link_draft (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  location                      nvarchar(255),
  when_publish                  datetime2,
  link_comment                  nvarchar(255),
  dirty                         integer default 0 not null,
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  deleted                       integer default 0 not null,
  constraint pk_link_draft primary key (id)
);
create sequence link_draft_seq as bigint start with 1 increment by 50;

create table la_attr_value (
  id                            integer not null,
  name                          nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_la_attr_value primary key (id)
);
create sequence la_attr_value_seq as bigint start with 1 increment by 50;

create table la_attr_value_attribute (
  la_attr_value_id              integer not null,
  attribute_id                  integer not null,
  constraint pk_la_attr_value_attribute primary key (la_attr_value_id,attribute_id)
);

create table looney (
  id                            numeric(19) not null,
  tune_id                       numeric(19),
  name                          nvarchar(255),
  constraint pk_looney primary key (id)
);
create sequence looney_seq as bigint start with 1 increment by 50;

create table m2_mcache_child (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_m2_mcache_child primary key (id)
);
create sequence m2_mcache_child_seq as bigint start with 1 increment by 50;

create table m2_mcache_master (
  id                            integer not null,
  constraint pk_m2_mcache_master primary key (id)
);
create sequence m2_mcache_master_seq as bigint start with 1 increment by 50;

create table m2mcache_set1 (
  m2_mcache_master_id           integer not null,
  m2_mcache_child_id            integer not null,
  constraint pk_m2mcache_set1 primary key (m2_mcache_master_id,m2_mcache_child_id)
);

create table m2mcache_set2 (
  m2_mcache_master_id           integer not null,
  m2_mcache_child_id            integer not null,
  constraint pk_m2mcache_set2 primary key (m2_mcache_master_id,m2_mcache_child_id)
);

create table maddress (
  id                            uniqueidentifier not null,
  street                        nvarchar(255),
  city                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_maddress primary key (id)
);

create table mcontact (
  id                            uniqueidentifier not null,
  email                         nvarchar(255),
  first_name                    nvarchar(255),
  last_name                     nvarchar(255),
  customer_id                   uniqueidentifier,
  version                       numeric(19) not null,
  constraint pk_mcontact primary key (id)
);

create table mcontact_message (
  id                            uniqueidentifier not null,
  title                         nvarchar(255),
  subject                       nvarchar(255),
  notes                         nvarchar(255),
  contact_id                    uniqueidentifier not null,
  version                       numeric(19) not null,
  constraint pk_mcontact_message primary key (id)
);

create table mcustomer (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  notes                         nvarchar(255),
  shipping_address_id           uniqueidentifier,
  billing_address_id            uniqueidentifier,
  version                       numeric(19) not null,
  constraint pk_mcustomer primary key (id)
);

create table mgroup (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_mgroup primary key (id)
);
create sequence mgroup_seq as bigint start with 1 increment by 50;

create table mmachine (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mmachine primary key (id)
);
create sequence mmachine_seq as bigint start with 1 increment by 50;

create table mmachine_mgroup (
  mmachine_id                   numeric(19) not null,
  mgroup_id                     numeric(19) not null,
  constraint pk_mmachine_mgroup primary key (mmachine_id,mgroup_id)
);

create table mmedia (
  type                          nvarchar(31) not null,
  id                            numeric(19) not null,
  url                           nvarchar(255),
  note                          nvarchar(255),
  constraint pk_mmedia primary key (id)
);
create sequence mmedia_seq as bigint start with 1 increment by 50;

create table non_updateprop (
  id                            integer not null,
  non_enum                      nvarchar(5),
  name                          nvarchar(255),
  note                          nvarchar(255),
  constraint ck_non_updateprop_non_enum check ( non_enum in ('BEGIN','END')),
  constraint pk_non_updateprop primary key (id)
);
create sequence non_updateprop_seq as bigint start with 1 increment by 50;

create table mprinter (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  flags                         numeric(19) not null,
  current_state_id              numeric(19),
  last_swap_cyan_id             numeric(19),
  last_swap_magenta_id          numeric(19),
  last_swap_yellow_id           numeric(19),
  last_swap_black_id            numeric(19),
  version                       numeric(19) not null,
  constraint pk_mprinter primary key (id)
);
create unique nonclustered index uq_mprinter_last_swap_cyan_id on mprinter(last_swap_cyan_id) where last_swap_cyan_id is not null;
create unique nonclustered index uq_mprinter_last_swap_magenta_id on mprinter(last_swap_magenta_id) where last_swap_magenta_id is not null;
create unique nonclustered index uq_mprinter_last_swap_yellow_id on mprinter(last_swap_yellow_id) where last_swap_yellow_id is not null;
create unique nonclustered index uq_mprinter_last_swap_black_id on mprinter(last_swap_black_id) where last_swap_black_id is not null;
create sequence mprinter_seq as bigint start with 1 increment by 50;

create table mprinter_state (
  id                            numeric(19) not null,
  flags                         numeric(19) not null,
  printer_id                    numeric(19),
  version                       numeric(19) not null,
  constraint pk_mprinter_state primary key (id)
);
create sequence mprinter_state_seq as bigint start with 1 increment by 50;

create table mprofile (
  id                            numeric(19) not null,
  picture_id                    numeric(19),
  name                          nvarchar(255),
  constraint pk_mprofile primary key (id)
);
create sequence mprofile_seq as bigint start with 1 increment by 50;

create table mprotected_construct_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_mprotected_construct_bean primary key (id)
);
create sequence mprotected_construct_bean_seq as bigint start with 1 increment by 50;

create table mrole (
  roleid                        integer not null,
  role_name                     nvarchar(255),
  constraint pk_mrole primary key (roleid)
);
create sequence mrole_seq as bigint start with 1 increment by 50;

create table mrole_muser (
  mrole_roleid                  integer not null,
  muser_userid                  integer not null,
  constraint pk_mrole_muser primary key (mrole_roleid,muser_userid)
);

create table msome_other (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_msome_other primary key (id)
);
create sequence msome_other_seq as bigint start with 1 increment by 50;

create table muser (
  userid                        integer not null,
  user_name                     nvarchar(255),
  user_type_id                  integer,
  constraint pk_muser primary key (userid)
);
create sequence muser_seq as bigint start with 1 increment by 50;

create table muser_type (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_muser_type primary key (id)
);
create sequence muser_type_seq as bigint start with 1 increment by 50;

create table mail_box (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mail_box primary key (id)
);
create sequence mail_box_seq as bigint start with 1 increment by 50;

create table mail_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mail_user primary key (id)
);
create sequence mail_user_seq as bigint start with 1 increment by 50;

create table mail_user_inbox (
  mail_user_id                  numeric(19) not null,
  mail_box_id                   numeric(19) not null,
  constraint pk_mail_user_inbox primary key (mail_user_id,mail_box_id)
);

create table mail_user_outbox (
  mail_user_id                  numeric(19) not null,
  mail_box_id                   numeric(19) not null,
  constraint pk_mail_user_outbox primary key (mail_user_id,mail_box_id)
);

create table main_entity (
  id                            nvarchar(255) not null,
  attr1                         nvarchar(255),
  attr2                         nvarchar(255),
  constraint pk_main_entity primary key (id)
);

create table main_entity_relation (
  id                            uniqueidentifier not null,
  id1                           nvarchar(255),
  id2                           nvarchar(255),
  attr1                         nvarchar(255),
  constraint pk_main_entity_relation primary key (id)
);

create table map_super_actual (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  when_created                  datetime2 not null,
  when_updated                  datetime2 not null,
  constraint pk_map_super_actual primary key (id)
);
create sequence map_super_actual_seq as bigint start with 1 increment by 50;

create table c_message (
  id                            numeric(19) not null,
  title                         nvarchar(255),
  body                          nvarchar(255),
  conversation_id               numeric(19),
  user_id                       numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_c_message primary key (id)
);
create sequence c_message_seq as bigint start with 1 increment by 50;

create table meter_address_data (
  id                            uniqueidentifier not null,
  street                        nvarchar(255) not null,
  constraint pk_meter_address_data primary key (id)
);

create table meter_contract_data (
  id                            uniqueidentifier not null,
  special_needs_client_id       uniqueidentifier not null,
  constraint uq_meter_contract_data_special_needs_client_id unique (special_needs_client_id),
  constraint pk_meter_contract_data primary key (id)
);

create table meter_special_needs_client (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  primary_id                    uniqueidentifier,
  constraint pk_meter_special_needs_client primary key (id)
);
create unique nonclustered index uq_meter_special_needs_client_primary_id on meter_special_needs_client(primary_id) where primary_id is not null;

create table meter_special_needs_contact (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_meter_special_needs_contact primary key (id)
);

create table meter_version (
  id                            uniqueidentifier not null,
  address_data_id               uniqueidentifier,
  contract_data_id              uniqueidentifier not null,
  constraint uq_meter_version_contract_data_id unique (contract_data_id),
  constraint pk_meter_version primary key (id)
);
create unique nonclustered index uq_meter_version_address_data_id on meter_version(address_data_id) where address_data_id is not null;

create table mnoc_role (
  role_id                       integer not null,
  role_name                     nvarchar(255),
  version                       integer not null,
  constraint pk_mnoc_role primary key (role_id)
);
create sequence mnoc_role_seq as bigint start with 1 increment by 50;

create table mnoc_user (
  user_id                       integer not null,
  user_name                     nvarchar(255),
  version                       integer not null,
  constraint pk_mnoc_user primary key (user_id)
);
create sequence mnoc_user_seq as bigint start with 1 increment by 50;

create table mnoc_user_mnoc_role (
  mnoc_user_user_id             integer not null,
  mnoc_role_role_id             integer not null,
  constraint pk_mnoc_user_mnoc_role primary key (mnoc_user_user_id,mnoc_role_role_id)
);

create table mny_a (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_mny_a primary key (id)
);
create sequence mny_a_seq as bigint start with 1 increment by 50;

create table mny_b (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  a_id                          numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_mny_b primary key (id)
);
create sequence mny_b_seq as bigint start with 1 increment by 50;

create table mny_b_mny_c (
  mny_b_id                      numeric(19) not null,
  mny_c_id                      numeric(19) not null,
  constraint pk_mny_b_mny_c primary key (mny_b_id,mny_c_id)
);

create table mny_c (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_mny_c primary key (id)
);
create sequence mny_c_seq as bigint start with 1 increment by 50;

create table mny_edge (
  id                            integer not null,
  from_id                       integer,
  to_id                         integer,
  flags                         integer not null,
  constraint pk_mny_edge primary key (id)
);
create unique nonclustered index uq_mny_edge_from_id_to_id on mny_edge(from_id,to_id) where from_id is not null and to_id is not null;
create unique nonclustered index uq_mny_edge_to_id_from_id on mny_edge(to_id,from_id) where to_id is not null and from_id is not null;
create sequence mny_edge_seq as bigint start with 1 increment by 50;

create table mny_node (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_mny_node primary key (id)
);
create sequence mny_node_seq as bigint start with 1 increment by 50;

create table mny_topic (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mny_topic primary key (id)
);
create sequence mny_topic_seq as bigint start with 1 increment by 50;

create table subtopics (
  topic                         numeric(19) not null,
  subtopic                      numeric(19) not null,
  constraint pk_subtopics primary key (topic,subtopic)
);

create table mp_role (
  id                            numeric(19) not null,
  mp_user_id                    numeric(19) not null,
  code                          nvarchar(255),
  organization_id               numeric(19),
  constraint pk_mp_role primary key (id)
);
create sequence mp_role_seq as bigint start with 1 increment by 50;

create table mp_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_mp_user primary key (id)
);
create sequence mp_user_seq as bigint start with 1 increment by 50;

create table ms_many_a (
  aid                           numeric(19) not null,
  name                          nvarchar(255),
  ms_many_a_many_b              integer default 0 not null,
  ms_many_b                     integer default 0 not null,
  deleted                       integer default 0 not null,
  constraint pk_ms_many_a primary key (aid)
);
create sequence ms_many_a_seq as bigint start with 1 increment by 50;

create table ms_many_a_many_b (
  ms_many_a_aid                 numeric(19) not null,
  ms_many_b_bid                 numeric(19) not null,
  constraint pk_ms_many_a_many_b primary key (ms_many_a_aid,ms_many_b_bid)
);

create table ms_many_b (
  bid                           numeric(19) not null,
  name                          nvarchar(255),
  deleted                       integer default 0 not null,
  constraint pk_ms_many_b primary key (bid)
);
create sequence ms_many_b_seq as bigint start with 1 increment by 50;

create table ms_many_b_many_a (
  ms_many_b_bid                 numeric(19) not null,
  ms_many_a_aid                 numeric(19) not null,
  constraint pk_ms_many_b_many_a primary key (ms_many_b_bid,ms_many_a_aid)
);

create table my_lob_size (
  id                            integer not null,
  name                          nvarchar(255),
  my_count                      integer not null,
  my_lob                        nvarchar(max),
  constraint pk_my_lob_size primary key (id)
);
create sequence my_lob_size_seq as bigint start with 1 increment by 50;

create table my_lob_size_join_many (
  id                            integer not null,
  something                     nvarchar(255),
  other                         nvarchar(255),
  parent_id                     integer,
  constraint pk_my_lob_size_join_many primary key (id)
);
create sequence my_lob_size_join_many_seq as bigint start with 1 increment by 50;

create table noidbean (
  name                          nvarchar(255),
  subject                       nvarchar(255),
  when_created                  datetime2 not null
);

create table o_bean_child (
  id                            numeric(19) not null,
  cached_bean_id                numeric(19),
  constraint pk_o_bean_child primary key (id)
);
create sequence o_bean_child_seq as bigint start with 1 increment by 50;

create table ocached_app (
  id                            numeric(19) not null,
  app_name                      nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ocached_app primary key (id)
);
create unique nonclustered index uq_ocached_app_app_name on ocached_app(app_name) where app_name is not null;
create sequence ocached_app_seq as bigint start with 1 increment by 50;

create table ocached_app_detail (
  id                            numeric(19) not null,
  app_id                        numeric(19) not null,
  detail                        nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ocached_app_detail primary key (id)
);
create unique nonclustered index uq_ocached_app_detail_app_id_detail on ocached_app_detail(app_id,detail) where detail is not null;
create sequence ocached_app_detail_seq as bigint start with 1 increment by 50;

create table o_cached_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_o_cached_bean primary key (id)
);
create sequence o_cached_bean_seq as bigint start with 1 increment by 50;

create table o_cached_bean_country (
  o_cached_bean_id              numeric(19) not null,
  o_country_code                nvarchar(2) not null,
  constraint pk_o_cached_bean_country primary key (o_cached_bean_id,o_country_code)
);

create table o_cached_bean_child (
  id                            numeric(19) not null,
  cached_bean_id                numeric(19),
  constraint pk_o_cached_bean_child primary key (id)
);
create sequence o_cached_bean_child_seq as bigint start with 1 increment by 50;

create table o_cached_inherit (
  dtype                         nvarchar(31) not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  child_adata                   nvarchar(255),
  child_bdata                   nvarchar(255),
  constraint pk_o_cached_inherit primary key (id)
);
create sequence o_cached_inherit_seq as bigint start with 1 increment by 50;

create table o_cached_natkey (
  id                            numeric(19) not null,
  store                         nvarchar(255),
  sku                           nvarchar(255),
  description                   nvarchar(255),
  constraint pk_o_cached_natkey primary key (id)
);
create sequence o_cached_natkey_seq as bigint start with 1 increment by 50;

create table o_cached_natkey3 (
  id                            numeric(19) not null,
  store                         nvarchar(255),
  code                          integer not null,
  sku                           nvarchar(255),
  description                   nvarchar(255),
  constraint pk_o_cached_natkey3 primary key (id)
);
create sequence o_cached_natkey3_seq as bigint start with 1 increment by 50;

create table ocached_nkey_uid (
  id                            numeric(19) not null,
  cid                           uniqueidentifier,
  other                         nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_ocached_nkey_uid primary key (id)
);
create sequence ocached_nkey_uid_seq as bigint start with 1 increment by 50;

create table ocar (
  id                            integer not null,
  vin                           nvarchar(255),
  name                          nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_ocar primary key (id)
);
create sequence ocar_seq as bigint start with 1 increment by 50;

create table ocompany (
  id                            integer not null,
  corp_id                       nvarchar(50),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_ocompany primary key (id)
);
create unique nonclustered index uq_ocompany_corp_id on ocompany(corp_id) where corp_id is not null;
create sequence ocompany_seq as bigint start with 1 increment by 50;

create table oengine (
  engine_id                     uniqueidentifier not null,
  short_desc                    nvarchar(255),
  car_id                        integer,
  version                       integer not null,
  constraint pk_oengine primary key (engine_id)
);
create unique nonclustered index uq_oengine_car_id on oengine(car_id) where car_id is not null;

create table ogear_box (
  id                            uniqueidentifier not null,
  box_desc                      nvarchar(255),
  box_size                      integer,
  car_id                        integer,
  version                       integer not null,
  constraint pk_ogear_box primary key (id)
);
create unique nonclustered index uq_ogear_box_car_id on ogear_box(car_id) where car_id is not null;

create table omvertex (
  id                            uniqueidentifier not null,
  constraint pk_omvertex primary key (id)
);

create table omvertex_other (
  id                            uniqueidentifier not null,
  omvertex_id                   uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_omvertex_other primary key (id)
);

create table oroad_show_msg (
  id                            integer not null,
  company_id                    integer not null,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint uq_oroad_show_msg_company_id unique (company_id),
  constraint pk_oroad_show_msg primary key (id)
);
create sequence oroad_show_msg_seq as bigint start with 1 increment by 50;

create table om_account_child_dbo (
  id                            numeric(19) not null,
  description                   nvarchar(255),
  banana_rama_id                numeric(19),
  constraint pk_om_account_child_dbo primary key (id)
);
create sequence om_account_child_dbo_seq as bigint start with 1 increment by 50;

create table om_account_dbo (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_om_account_dbo primary key (id)
);
create sequence om_account_dbo_seq as bigint start with 1 increment by 50;

create table om_basic_child (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  parent_id                     numeric(19),
  version                       numeric(19) not null,
  constraint pk_om_basic_child primary key (id)
);
create sequence om_basic_child_seq as bigint start with 1 increment by 50;

create table om_basic_parent (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_om_basic_parent primary key (id)
);
create sequence om_basic_parent_seq as bigint start with 1 increment by 50;

create table om_bean_list_child (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  parent_id                     numeric(19),
  version                       numeric(19) not null,
  constraint pk_om_bean_list_child primary key (id)
);
create sequence om_bean_list_child_seq as bigint start with 1 increment by 50;

create table om_bean_list_parent (
  id                            numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_om_bean_list_parent primary key (id)
);
create sequence om_bean_list_parent_seq as bigint start with 1 increment by 50;

create table om_ordered_detail (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  master_id                     numeric(19),
  version                       numeric(19) not null,
  sort_order                    integer,
  constraint pk_om_ordered_detail primary key (id)
);
create sequence om_ordered_detail_seq as bigint start with 1 increment by 50;

create table om_ordered_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_om_ordered_master primary key (id)
);
create sequence om_ordered_master_seq as bigint start with 1 increment by 50;

create table oml_bar (
  id                            numeric(19) not null,
  constraint pk_oml_bar primary key (id)
);
create sequence oml_bar_seq as bigint start with 1 increment by 50;

create table oml_baz (
  id                            numeric(19) not null,
  foo_id                        numeric(19) not null,
  constraint pk_oml_baz primary key (id)
);
create sequence oml_baz_seq as bigint start with 1 increment by 50;

create table oml_foo (
  id                            numeric(19) not null,
  bar_id                        numeric(19) not null,
  constraint pk_oml_foo primary key (id)
);
create sequence oml_foo_seq as bigint start with 1 increment by 50;

create table only_id_entity (
  id                            numeric(19) not null,
  constraint pk_only_id_entity primary key (id)
);
create sequence only_id_entity_seq as bigint start with 1 increment by 50;

create table o_order (
  id                            integer not null,
  status                        integer,
  order_date                    date,
  ship_date                     date,
  kcustomer_id                  integer not null,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  constraint ck_o_order_status check ( status in (0,1,2,3)),
  constraint pk_o_order primary key (id)
);
create sequence o_order_seq as bigint start with 1 increment by 50;

create table o_order_detail (
  id                            integer not null,
  order_id                      integer not null,
  order_qty                     integer,
  ship_qty                      integer,
  unit_price                    float(32),
  product_id                    integer,
  cretime                       datetime2,
  updtime                       datetime2 not null,
  constraint pk_o_order_detail primary key (id)
);
create sequence o_order_detail_seq as bigint start with 1 increment by 50;

create table s_orders (
  uuid                          nvarchar(40) not null,
  constraint pk_s_orders primary key (uuid)
);

create table s_order_items (
  uuid                          nvarchar(40) not null,
  product_variant_uuid          nvarchar(255),
  order_uuid                    nvarchar(40),
  quantity                      integer not null,
  amount                        numeric(16,3),
  constraint pk_s_order_items primary key (uuid)
);

create table order_master (
  id                            numeric(19) not null,
  constraint pk_order_master primary key (id)
);
create sequence order_master_seq as bigint start with 1 increment by 50;

create table order_master_inheritance (
  id                            integer not null,
  constraint pk_order_master_inheritance primary key (id)
);
create sequence order_master_inheritance_seq as bigint start with 1 increment by 50;

create table order_referenced_parent (
  type                          nvarchar(31) not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  child_name                    nvarchar(255),
  master_id                     numeric(19),
  sort_order                    integer,
  constraint pk_order_referenced_parent primary key (id)
);
create sequence order_referenced_parent_seq as bigint start with 1 increment by 50;

create table or_order_ship (
  id                            integer not null,
  order_id                      integer,
  ship_time                     datetime2,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_or_order_ship primary key (id)
);
create sequence or_order_ship_seq as bigint start with 1 increment by 50;

create table order_toy (
  id                            integer not null,
  title                         nvarchar(255),
  child_id                      numeric(19),
  sort_order                    integer,
  constraint pk_order_toy primary key (id)
);
create sequence order_toy_seq as bigint start with 1 increment by 50;

create table ordered_parent (
  dtype                         nvarchar(31) not null,
  id                            integer not null,
  order_master_inheritance_id   integer not null,
  common_name                   nvarchar(255),
  sort_order                    integer,
  ordered_aname                 nvarchar(255),
  ordered_bname                 nvarchar(255),
  constraint pk_ordered_parent primary key (id)
);
create sequence ordered_parent_seq as bigint start with 1 increment by 50;

create table organisation (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_organisation primary key (id)
);
create sequence organisation_seq as bigint start with 1 increment by 50;

create table organization_node (
  kind                          nvarchar(31) not null,
  id                            numeric(19) not null,
  parent_tree_node_id           numeric(19) not null,
  title                         nvarchar(255),
  constraint uq_organization_node_parent_tree_node_id unique (parent_tree_node_id),
  constraint pk_organization_node primary key (id)
);
create sequence organization_node_seq as bigint start with 1 increment by 50;

create table organization_tree_node (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_organization_tree_node primary key (id)
);
create sequence organization_tree_node_seq as bigint start with 1 increment by 50;

create table orp_detail (
  id                            nvarchar(100) not null,
  detail                        nvarchar(255),
  master_id                     nvarchar(100),
  version                       numeric(19) not null,
  constraint pk_orp_detail primary key (id)
);

create table orp_detail2 (
  id                            nvarchar(100) not null,
  orp_master2_id                nvarchar(100) not null,
  detail                        nvarchar(255),
  master_id                     nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_orp_detail2 primary key (id)
);

create table orp_master (
  id                            nvarchar(100) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_orp_master primary key (id)
);

create table orp_master2 (
  id                            nvarchar(100) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_orp_master2 primary key (id)
);

create table oto_aone (
  id                            nvarchar(100) not null,
  description                   nvarchar(255),
  constraint pk_oto_aone primary key (id)
);

create table oto_atwo (
  id                            nvarchar(100) not null,
  description                   nvarchar(255),
  aone_id                       nvarchar(100),
  constraint pk_oto_atwo primary key (id)
);
create unique nonclustered index uq_oto_atwo_aone_id on oto_atwo(aone_id) where aone_id is not null;

create table oto_bchild (
  master_id                     numeric(19) not null,
  child                         nvarchar(255),
  constraint pk_oto_bchild primary key (master_id)
);
create sequence oto_bchild_seq as bigint start with 1 increment by 50;

create table oto_bmaster (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_oto_bmaster primary key (id)
);
create sequence oto_bmaster_seq as bigint start with 1 increment by 50;

create table oto_child (
  id                            integer not null,
  name                          nvarchar(255),
  master_id                     numeric(19),
  constraint pk_oto_child primary key (id)
);
create unique nonclustered index uq_oto_child_master_id on oto_child(master_id) where master_id is not null;
create sequence oto_child_seq as bigint start with 1 increment by 50;

create table oto_cust (
  cid                           numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_cust primary key (cid)
);
create sequence oto_cust_seq as bigint start with 1 increment by 50;

create table oto_cust_address (
  aid                           numeric(19) not null,
  line1                         nvarchar(255),
  line2                         nvarchar(255),
  line3                         nvarchar(255),
  customer_cid                  numeric(19),
  version                       numeric(19) not null,
  constraint pk_oto_cust_address primary key (aid)
);
create unique nonclustered index uq_oto_cust_address_customer_cid on oto_cust_address(customer_cid) where customer_cid is not null;
create sequence oto_cust_address_seq as bigint start with 1 increment by 50;

create table oto_level_a (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  b_id                          numeric(19),
  constraint pk_oto_level_a primary key (id)
);
create unique nonclustered index uq_oto_level_a_b_id on oto_level_a(b_id) where b_id is not null;
create sequence oto_level_a_seq as bigint start with 1 increment by 50;

create table oto_level_b (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  c_id                          numeric(19),
  constraint pk_oto_level_b primary key (id)
);
create unique nonclustered index uq_oto_level_b_c_id on oto_level_b(c_id) where c_id is not null;
create sequence oto_level_b_seq as bigint start with 1 increment by 50;

create table oto_level_c (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_oto_level_c primary key (id)
);
create sequence oto_level_c_seq as bigint start with 1 increment by 50;

create table oto_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_oto_master primary key (id)
);
create sequence oto_master_seq as bigint start with 1 increment by 50;

create table oto_prime (
  pid                           numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_prime primary key (pid)
);
create sequence oto_prime_seq as bigint start with 1 increment by 50;

create table oto_prime_extra (
  eid                           numeric(19) not null,
  extra                         nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_prime_extra primary key (eid)
);
create sequence oto_prime_extra_seq as bigint start with 1 increment by 50;

create table oto_sd_child (
  id                            numeric(19) not null,
  child                         nvarchar(255),
  master_id                     numeric(19),
  deleted                       integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_oto_sd_child primary key (id)
);
create unique nonclustered index uq_oto_sd_child_master_id on oto_sd_child(master_id) where master_id is not null;
create sequence oto_sd_child_seq as bigint start with 1 increment by 50;

create table oto_sd_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_sd_master primary key (id)
);
create sequence oto_sd_master_seq as bigint start with 1 increment by 50;

create table oto_th_many (
  id                            numeric(19) not null,
  oto_th_top_id                 numeric(19) not null,
  many                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_th_many primary key (id)
);
create sequence oto_th_many_seq as bigint start with 1 increment by 50;

create table oto_th_one (
  id                            numeric(19) not null,
  one                           integer default 0 not null,
  many_id                       numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_th_one primary key (id)
);
create unique nonclustered index uq_oto_th_one_many_id on oto_th_one(many_id) where many_id is not null;
create sequence oto_th_one_seq as bigint start with 1 increment by 50;

create table oto_th_top (
  id                            numeric(19) not null,
  topp                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_th_top primary key (id)
);
create sequence oto_th_top_seq as bigint start with 1 increment by 50;

create table oto_ubprime (
  pid                           uniqueidentifier not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_ubprime primary key (pid)
);

create table oto_ubprime_extra (
  eid                           uniqueidentifier not null,
  extra                         nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_ubprime_extra primary key (eid)
);

create table oto_uprime (
  pid                           uniqueidentifier not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_uprime primary key (pid)
);

create table oto_uprime_extra (
  eid                           uniqueidentifier not null,
  extra                         nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_oto_uprime_extra primary key (eid)
);

create table oto_user_model (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  user_optional_id              numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_user_model primary key (id)
);
create unique nonclustered index uq_oto_user_model_user_optional_id on oto_user_model(user_optional_id) where user_optional_id is not null;
create sequence oto_user_model_seq as bigint start with 1 increment by 50;

create table oto_user_model_optional (
  id                            numeric(19) not null,
  optional                      nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_oto_user_model_optional primary key (id)
);
create sequence oto_user_model_optional_seq as bigint start with 1 increment by 50;

create table pfile (
  id                            integer not null,
  name                          nvarchar(255),
  file_content_id               integer,
  file_content2_id              integer,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_pfile primary key (id)
);
create unique nonclustered index uq_pfile_file_content_id on pfile(file_content_id) where file_content_id is not null;
create unique nonclustered index uq_pfile_file_content2_id on pfile(file_content2_id) where file_content2_id is not null;
create sequence pfile_seq as bigint start with 1 increment by 50;

create table pfile_content (
  id                            integer not null,
  content                       image,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_pfile_content primary key (id)
);
create sequence pfile_content_seq as bigint start with 1 increment by 50;

create table paggview (
  pview_id                      uniqueidentifier,
  amount                        integer not null
);
create unique nonclustered index uq_paggview_pview_id on paggview(pview_id) where pview_id is not null;

create table pallet_location (
  type                          nvarchar(31) not null,
  id                            integer not null,
  zone_sid                      integer not null,
  attribute                     nvarchar(255),
  constraint pk_pallet_location primary key (id)
);
create sequence pallet_location_seq as bigint start with 1 increment by 50;

create table parcel (
  parcelid                      numeric(19) not null,
  description                   nvarchar(255),
  constraint pk_parcel primary key (parcelid)
);
create sequence parcel_seq as bigint start with 1 increment by 50;

create table parcel_location (
  parcellocid                   numeric(19) not null,
  location                      nvarchar(255),
  parcelid                      numeric(19),
  constraint pk_parcel_location primary key (parcellocid)
);
create unique nonclustered index uq_parcel_location_parcelid on parcel_location(parcelid) where parcelid is not null;
create sequence parcel_location_seq as bigint start with 1 increment by 50;

create table rawinherit_parent (
  type                          nvarchar(31) not null,
  id                            numeric(19) not null,
  val                           integer,
  more                          nvarchar(255),
  constraint pk_rawinherit_parent primary key (id)
);
create sequence rawinherit_parent_seq as bigint start with 1 increment by 50;

create table rawinherit_parent_rawinherit_data (
  rawinherit_parent_id          numeric(19) not null,
  rawinherit_data_id            numeric(19) not null,
  constraint pk_rawinherit_parent_rawinherit_data primary key (rawinherit_parent_id,rawinherit_data_id)
);

create table e_save_test_c (
  id                            numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_e_save_test_c primary key (id)
);
create sequence e_save_test_c_seq as bigint start with 1 increment by 50;

create table parent_a (
  id                            numeric(19) not null,
  child_id                      numeric(19),
  name                          nvarchar(255),
  constraint pk_parent_a primary key (id)
);
create sequence parent_a_seq as bigint start with 1 increment by 50;

create table parent_b (
  id                            numeric(19) not null,
  child_id                      numeric(19),
  constraint pk_parent_b primary key (id)
);
create sequence parent_b_seq as bigint start with 1 increment by 50;

create table parent_person (
  identifier                    integer not null,
  name                          nvarchar(255),
  age                           integer,
  some_bean_id                  integer,
  parent_identifier             integer,
  family_name                   nvarchar(255),
  address                       nvarchar(255),
  constraint pk_parent_person primary key (identifier)
);
create sequence parent_person_seq as bigint start with 1 increment by 50;

create table c_participation (
  id                            numeric(19) not null,
  rating                        integer,
  type                          integer,
  conversation_id               numeric(19) not null,
  user_id                       numeric(19) not null,
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint ck_c_participation_type check ( type in (0,1)),
  constraint pk_c_participation primary key (id)
);
create sequence c_participation_seq as bigint start with 1 increment by 50;

create table password_store_model (
  id                            numeric(19) not null,
  enc1                          nvarchar(30),
  enc2                          nvarchar(40),
  enc3                          nvarchar(max),
  enc4                          varbinary(30),
  enc5                          varbinary(40),
  enc6                          image,
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_password_store_model primary key (id)
);
create sequence password_store_model_seq as bigint start with 1 increment by 50;

create table pcf_calendar (
  id                            numeric(19) not null,
  pcf_person_id                 numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_pcf_calendar primary key (id)
);
create sequence pcf_calendar_seq as bigint start with 1 increment by 50;

create table pcf_city (
  id                            numeric(19) not null,
  pcf_country_id                numeric(19) not null,
  name                          nvarchar(255),
  mayor_id                      numeric(19) not null,
  vice_mayor_id                 numeric(19) not null,
  version                       numeric(19) not null,
  constraint uq_pcf_city_mayor_id unique (mayor_id),
  constraint uq_pcf_city_vice_mayor_id unique (vice_mayor_id),
  constraint pk_pcf_city primary key (id)
);
create sequence pcf_city_seq as bigint start with 1 increment by 50;

create table pcf_country (
  id                            numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_pcf_country primary key (id)
);
create sequence pcf_country_seq as bigint start with 1 increment by 50;

create table pcf_event (
  id                            numeric(19) not null,
  pcf_calendar_id               numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_pcf_event primary key (id)
);
create sequence pcf_event_seq as bigint start with 1 increment by 50;

create table pcf_person (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_pcf_person primary key (id)
);
create sequence pcf_person_seq as bigint start with 1 increment by 50;

create table mt_permission (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_mt_permission primary key (id)
);

create table persistent_file (
  id                            integer not null,
  name                          nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_persistent_file primary key (id)
);
create sequence persistent_file_seq as bigint start with 1 increment by 50;

create table persistent_file_content (
  id                            integer not null,
  persistent_file_id            integer,
  content                       image,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_persistent_file_content primary key (id)
);
create unique nonclustered index uq_persistent_file_content_persistent_file_id on persistent_file_content(persistent_file_id) where persistent_file_id is not null;
create sequence persistent_file_content_seq as bigint start with 1 increment by 50;

create table person (
  oid                           numeric(19) not null,
  default_address_oid           numeric(19),
  version                       integer not null,
  constraint pk_person primary key (oid)
);
create sequence person_seq as bigint start with 1 increment by 50;

create table persons (
  id                            numeric(19) not null,
  surname                       nvarchar(64) not null,
  name                          nvarchar(64) not null,
  constraint pk_persons primary key (id)
);
create sequence PERSONS_seq as bigint start with 1000 increment by 40;

create table person_cache_email (
  id                            nvarchar(128) not null,
  person_info_person_id         nvarchar(128),
  email                         nvarchar(255),
  constraint pk_person_cache_email primary key (id)
);

create table person_cache_info (
  person_id                     nvarchar(128) not null,
  name                          nvarchar(255),
  constraint pk_person_cache_info primary key (person_id)
);

create table person_other (
  id                            nvarchar(128) not null,
  email                         nvarchar(255),
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_person_other primary key (id)
);

create table persona (
  id                            numeric(19) not null,
  persona                       nvarchar(255),
  person_oid                    numeric(19),
  version                       integer not null,
  constraint pk_persona primary key (id)
);
create unique nonclustered index uq_persona_person_oid on persona(person_oid) where person_oid is not null;
create sequence persona_seq as bigint start with 1 increment by 50;

create table phones (
  id                            numeric(19) not null,
  phone_number                  nvarchar(7) not null,
  person_id                     numeric(19) not null,
  constraint uq_phones_phone_number unique (phone_number),
  constraint pk_phones primary key (id)
);
create sequence PHONES_seq as bigint start with 1 increment by 50;

create table e_position (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  contract_id                   numeric(19) not null,
  constraint pk_e_position primary key (id)
);
create sequence e_position_seq as bigint start with 1 increment by 50;

create table primary_revision (
  id                            numeric(19) not null,
  revision                      integer not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_primary_revision primary key (id,revision)
);

create table o_product (
  id                            integer not null,
  sku                           nvarchar(20),
  name                          nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  constraint pk_o_product primary key (id)
);
create sequence o_product_seq as bigint start with 1 increment by 50;

create table pp (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  value                         nvarchar(100) not null,
  constraint pk_pp primary key (id)
);

create table pp_to_ww (
  pp_id                         uniqueidentifier not null,
  ww_id                         uniqueidentifier not null,
  constraint pk_pp_to_ww primary key (pp_id,ww_id)
);

create table question (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  groupobjectid                 numeric(19),
  sequence_number               integer not null,
  constraint pk_question primary key (id)
);
create sequence question_seq as bigint start with 1 increment by 50;

create table rcustomer (
  company                       nvarchar(127) not null,
  name                          nvarchar(127) not null,
  description                   nvarchar(255),
  constraint pk_rcustomer primary key (company,name)
);

create table r_orders (
  company                       nvarchar(127) not null,
  order_number                  integer not null,
  customername                  nvarchar(127),
  item                          nvarchar(255),
  constraint pk_r_orders primary key (company,order_number)
);

create table referenced_defaults_model (
  id                            integer not null,
  defaults_model_id             integer not null,
  name                          nvarchar(255),
  constraint pk_referenced_defaults_model primary key (id)
);
create sequence referenced_defaults_model_seq as bigint start with 1 increment by 50;

create table referenced_defaults_model_draft (
  id                            integer not null,
  defaults_model_id             integer not null,
  name                          nvarchar(255),
  constraint pk_referenced_defaults_model_draft primary key (id)
);
create sequence referenced_defaults_model_draft_seq as bigint start with 1 increment by 50;

create table referencing_bean (
  id                            uniqueidentifier not null,
  constraint pk_referencing_bean primary key (id)
);

create table region (
  customer                      integer not null,
  type                          integer not null,
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_region primary key (customer,type)
);

create table rel_detail (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_rel_detail primary key (id)
);
create sequence rel_detail_seq as bigint start with 1 increment by 50;

create table rel_master (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  detail_id                     numeric(19),
  version                       integer not null,
  constraint pk_rel_master primary key (id)
);
create sequence rel_master_seq as bigint start with 1 increment by 50;

create table app_config_re (
  id                            integer not null,
  config_id                     integer,
  constraint pk_app_config_re primary key (id)
);
create sequence app_config_re_seq as bigint start with 1 increment by 50;

create table resourcefile (
  id                            nvarchar(64) not null,
  parentresourcefileid          nvarchar(64),
  name                          nvarchar(128) not null,
  constraint pk_resourcefile primary key (id)
);

create table mt_role (
  id                            uniqueidentifier not null,
  name                          nvarchar(50),
  tenant_id                     uniqueidentifier,
  version                       numeric(19) not null,
  constraint pk_mt_role primary key (id)
);

create table mt_role_permission (
  mt_role_id                    uniqueidentifier not null,
  mt_permission_id              uniqueidentifier not null,
  constraint pk_mt_role_permission primary key (mt_role_id,mt_permission_id)
);

create table em_role (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_em_role primary key (id)
);
create sequence em_role_seq as bigint start with 1 increment by 50;

create table root_bean (
  dtype                         nvarchar(31) not null,
  id                            uniqueidentifier not null,
  referencing_bean_id           uniqueidentifier not null,
  value                         nvarchar(255),
  constraint pk_root_bean primary key (id)
);

create table f_second (
  id                            numeric(19) not null,
  mod_name                      nvarchar(255),
  first                         numeric(19),
  title                         nvarchar(255),
  constraint pk_f_second primary key (id)
);
create unique nonclustered index uq_f_second_first on f_second(first) where first is not null;
create sequence f_second_seq as bigint start with 1 increment by 50;

create table section (
  id                            integer not null,
  article_id                    integer,
  type                          integer,
  content                       nvarchar(max),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint ck_section_type check ( type in (0,1)),
  constraint pk_section primary key (id)
);
create sequence section_seq as bigint start with 1 increment by 50;

create table self_many_many (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_self_many_many primary key (id)
);
create sequence self_many_many_seq as bigint start with 1 increment by 50;

create table self_many_bridge (
  self_1_id                     numeric(19) not null,
  self_2_id                     numeric(19) not null,
  constraint pk_self_many_bridge primary key (self_1_id,self_2_id)
);

create table self_parent (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  parent_id                     numeric(19),
  version                       numeric(19) not null,
  constraint pk_self_parent primary key (id)
);
create sequence self_parent_seq as bigint start with 1 increment by 50;

create table self_ref_customer (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  referred_by_id                numeric(19),
  constraint pk_self_ref_customer primary key (id)
);
create sequence self_ref_customer_seq as bigint start with 1 increment by 50;

create table self_ref_example (
  id                            numeric(19) not null,
  name                          nvarchar(255) not null,
  parent_id                     numeric(19),
  constraint pk_self_ref_example primary key (id)
);
create sequence self_ref_example_seq as bigint start with 1 increment by 50;

create table e_save_test_a (
  id                            numeric(19) not null,
  version                       numeric(19) not null,
  constraint pk_e_save_test_a primary key (id)
);
create sequence e_save_test_a_seq as bigint start with 1 increment by 50;

create table e_save_test_b (
  id                            numeric(19) not null,
  sibling_a_id                  numeric(19),
  test_property                 integer default 0 not null,
  version                       numeric(19) not null,
  constraint pk_e_save_test_b primary key (id)
);
create unique nonclustered index uq_e_save_test_b_sibling_a_id on e_save_test_b(sibling_a_id) where sibling_a_id is not null;
create sequence e_save_test_b_seq as bigint start with 1 increment by 50;

create table site (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  parent_id                     uniqueidentifier,
  data_container_id             uniqueidentifier,
  site_address_id               uniqueidentifier,
  constraint pk_site primary key (id)
);
create unique nonclustered index uq_site_data_container_id on site(data_container_id) where data_container_id is not null;
create unique nonclustered index uq_site_site_address_id on site(site_address_id) where site_address_id is not null;

create table site_address (
  id                            uniqueidentifier not null,
  street                        nvarchar(255),
  city                          nvarchar(255),
  zip_code                      nvarchar(255),
  constraint pk_site_address primary key (id)
);

create table some_enum_bean (
  id                            numeric(19) not null,
  some_enum                     integer,
  name                          nvarchar(255),
  constraint ck_some_enum_bean_some_enum check ( some_enum in (0,1)),
  constraint pk_some_enum_bean primary key (id)
);
create sequence some_enum_bean_seq as bigint start with 1 increment by 50;

create table some_file_bean (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  content                       image,
  version                       numeric(19) not null,
  constraint pk_some_file_bean primary key (id)
);
create sequence some_file_bean_seq as bigint start with 1 increment by 50;

create table some_new_types_bean (
  id                            numeric(19) not null,
  dow                           integer,
  mth                           integer,
  yr                            integer,
  yr_mth                        date,
  month_day                     date,
  sql_date                      date,
  sql_time                      time,
  local_date                    date,
  local_date_time               datetime2,
  offset_date_time              datetime2,
  zoned_date_time               datetime2,
  local_time                    time,
  instant                       datetime2,
  zone_id                       nvarchar(60),
  zone_offset                   nvarchar(60),
  path                          nvarchar(255),
  period                        nvarchar(20),
  duration                      numeric(19),
  version                       numeric(19) not null,
  constraint ck_some_new_types_bean_dow check ( dow in (1,2,3,4,5,6,7)),
  constraint ck_some_new_types_bean_mth check ( mth in (1,2,3,4,5,6,7,8,9,10,11,12)),
  constraint pk_some_new_types_bean primary key (id)
);
create sequence some_new_types_bean_seq as bigint start with 1 increment by 50;

create table some_period_bean (
  id                            numeric(19) not null,
  anniversary                   date,
  version                       numeric(19) not null,
  constraint pk_some_period_bean primary key (id)
);
create sequence some_period_bean_seq as bigint start with 1 increment by 50;

create table source_base (
  dtype                         nvarchar(31) not null,
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  pos                           integer not null,
  target_id                     uniqueidentifier,
  constraint pk_source_base primary key (id)
);

create table stockforecast (
  type                          nvarchar(31) not null,
  id                            numeric(19) not null,
  inner_report_id               numeric(19),
  constraint pk_stockforecast primary key (id)
);
create sequence stockforecast_seq as bigint start with 1 increment by 50;

create table sub_section (
  id                            integer not null,
  section_id                    integer,
  title                         nvarchar(255),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_sub_section primary key (id)
);
create sequence sub_section_seq as bigint start with 1 increment by 50;

create table sub_type (
  sub_type_id                   integer not null,
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_sub_type primary key (sub_type_id)
);
create sequence sub_type_seq as bigint start with 1 increment by 50;

create table survey (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_survey primary key (id)
);
create sequence survey_seq as bigint start with 1 increment by 50;

create table tbytes_only (
  id                            integer not null,
  content                       image,
  constraint pk_tbytes_only primary key (id)
);
create sequence tbytes_only_seq as bigint start with 1 increment by 50;

create table tcar (
  type                          nvarchar(31) not null,
  plate_no                      nvarchar(32) not null,
  deleted                       integer default 0 not null,
  truckload                     numeric(19),
  constraint pk_tcar primary key (plate_no)
);

create table tevent (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_tevent primary key (id)
);
create sequence tevent_seq as bigint start with 1 increment by 50;

create table tevent_many (
  id                            numeric(19) not null,
  description                   nvarchar(255),
  event_id                      numeric(19),
  my_units                      integer not null,
  amount                        float(32) not null,
  version                       numeric(19) not null,
  constraint pk_tevent_many primary key (id)
);
create sequence tevent_many_seq as bigint start with 1 increment by 50;

create table tevent_one (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  status                        integer,
  event_id                      numeric(19),
  version                       numeric(19) not null,
  constraint ck_tevent_one_status check ( status in (0,1)),
  constraint pk_tevent_one primary key (id)
);
create unique nonclustered index uq_tevent_one_event_id on tevent_one(event_id) where event_id is not null;
create sequence tevent_one_seq as bigint start with 1 increment by 50;

create table tint_root (
  my_type                       integer not null,
  id                            integer not null,
  name                          nvarchar(255),
  child_property                nvarchar(255),
  constraint pk_tint_root primary key (id)
);
create sequence tint_root_seq as bigint start with 1 increment by 50;

create table tjoda_entity (
  id                            integer not null,
  local_time                    time,
  constraint pk_tjoda_entity primary key (id)
);
create sequence tjoda_entity_seq as bigint start with 1 increment by 50;

create table t_mapsuper1 (
  id                            integer not null,
  something                     nvarchar(255),
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_t_mapsuper1 primary key (id)
);
create sequence t_mapsuper1_seq as bigint start with 1 increment by 50;

create table t_oneb (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  active                        integer default 0 not null,
  constraint pk_t_oneb primary key (id)
);
create sequence t_oneb_seq as bigint start with 1 increment by 50;

create table t_detail_with_other_namexxxyy (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  some_unique_value             nvarchar(127),
  active                        integer default 0 not null,
  master_id                     integer,
  constraint pk_t_detail_with_other_namexxxyy primary key (id)
);
create unique nonclustered index uq_t_detail_with_other_namexxxyy_some_unique_value on t_detail_with_other_namexxxyy(some_unique_value) where some_unique_value is not null;
create sequence t_atable_detail_seq as bigint start with 1 increment by 50;

create table t_atable_thatisrelatively (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  active                        integer default 0 not null,
  constraint pk_t_atable_thatisrelatively primary key (id)
);
create sequence t_atable_master_seq as bigint start with 1 increment by 50;

create table ttruck_holder (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  truck_plate_no                nvarchar(32) not null,
  basic_id                      integer,
  version                       numeric(19) not null,
  constraint pk_ttruck_holder primary key (id)
);
create sequence ttruck_holder_seq as bigint start with 1 increment by 50;

create table ttruck_holder_item (
  id                            numeric(19) not null,
  some_uid                      uniqueidentifier,
  foo                           nvarchar(255),
  owner_id                      numeric(19) not null,
  constraint pk_ttruck_holder_item primary key (id)
);
create sequence ttruck_holder_item_seq as bigint start with 1 increment by 50;

create table tuuid_entity (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_tuuid_entity primary key (id)
);

create table twheel (
  id                            numeric(19) not null,
  owner_plate_no                nvarchar(32) not null,
  constraint pk_twheel primary key (id)
);
create sequence twheel_seq as bigint start with 1 increment by 50;

create table twith_pre_insert (
  id                            integer not null,
  name                          nvarchar(255) not null,
  title                         nvarchar(255),
  constraint pk_twith_pre_insert primary key (id)
);
create sequence twith_pre_insert_seq as bigint start with 1 increment by 50;

create table target_base (
  dtype                         nvarchar(31) not null,
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  constraint pk_target_base primary key (id)
);

create table mt_tenant (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_mt_tenant primary key (id)
);

create table test_annotation_base_entity (
  direct                        nvarchar(255),
  meta                          nvarchar(255),
  mixed                         nvarchar(255),
  constraint_annotation         nvarchar(40),
  null1                         nvarchar(255) not null,
  null2                         nvarchar(255),
  null3                         nvarchar(255)
);

create table test_jakarta_annotation_base_entity (
  direct                        nvarchar(255),
  meta                          nvarchar(255),
  mixed                         nvarchar(255),
  constraint_annotation         nvarchar(40),
  null1                         nvarchar(255) not null,
  null2                         nvarchar(255),
  null3                         nvarchar(255)
);

create table tire (
  id                            numeric(19) not null,
  wheel                         numeric(19),
  version                       integer not null,
  constraint pk_tire primary key (id)
);
create unique nonclustered index uq_tire_wheel on tire(wheel) where wheel is not null;
create sequence tire_seq as bigint start with 1 increment by 50;

create table sa_tire (
  id                            numeric(19) not null,
  version                       integer not null,
  constraint pk_sa_tire primary key (id)
);
create sequence sa_tire_seq as bigint start with 1 increment by 50;

create table tree_entity (
  id                            integer not null,
  text                          nvarchar(255),
  parent_id                     integer,
  constraint pk_tree_entity primary key (id)
);
create sequence tree_entity_seq as bigint start with 1 increment by 50;

create table trip (
  id                            integer not null,
  vehicle_driver_id             integer,
  destination                   nvarchar(255),
  address_id                    integer,
  star_date                     datetime2,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_trip primary key (id)
);
create sequence trip_seq as bigint start with 1 increment by 50;

create table truck_ref (
  id                            integer not null,
  something                     nvarchar(255),
  constraint pk_truck_ref primary key (id)
);
create sequence truck_ref_seq as bigint start with 1 increment by 50;

create table tune (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_tune primary key (id)
);
create sequence tune_seq as bigint start with 1 increment by 50;

create table [type] (
  customer                      integer not null,
  type                          integer not null,
  description                   nvarchar(255),
  sub_type_id                   integer,
  version                       numeric(19) not null,
  constraint pk_type primary key (customer,type)
);

create table tz_bean (
  id                            numeric(19) not null,
  moda                          nvarchar(255),
  ts                            datetime2,
  tstz                          datetime2,
  constraint pk_tz_bean primary key (id)
);
create sequence tz_bean_seq as bigint start with 1 increment by 50;

create table usib_child (
  id                            uniqueidentifier not null,
  parent_id                     numeric(19),
  deleted                       integer default 0 not null,
  constraint pk_usib_child primary key (id)
);

create table usib_child_sibling (
  id                            numeric(19) not null,
  child_id                      uniqueidentifier,
  deleted                       integer default 0 not null,
  constraint pk_usib_child_sibling primary key (id)
);
create unique nonclustered index uq_usib_child_sibling_child_id on usib_child_sibling(child_id) where child_id is not null;
create sequence usib_child_sibling_seq as bigint start with 1 increment by 50;

create table usib_parent (
  id                            numeric(19) not null,
  deleted                       integer default 0 not null,
  constraint pk_usib_parent primary key (id)
);
create sequence usib_parent_seq as bigint start with 1 increment by 50;

create table ut_detail (
  id                            integer not null,
  utmaster_id                   integer not null,
  name                          nvarchar(255),
  qty                           integer,
  amount                        float(32),
  version                       integer not null,
  constraint pk_ut_detail primary key (id)
);
create sequence ut_detail_seq as bigint start with 1 increment by 50;

create table ut_master (
  id                            integer not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  event_date                    date,
  version                       integer not null,
  constraint pk_ut_master primary key (id)
);
create sequence ut_master_seq as bigint start with 1 increment by 50;

create table uuone (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  description                   nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_uuone primary key (id)
);

create table uutwo (
  id                            uniqueidentifier not null,
  name                          nvarchar(255),
  notes                         nvarchar(255),
  master_id                     uniqueidentifier,
  version                       numeric(19) not null,
  constraint pk_uutwo primary key (id)
);

create table oto_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  account_id                    numeric(19) not null,
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint uq_oto_user_account_id unique (account_id),
  constraint pk_oto_user primary key (id)
);
create sequence oto_user_seq as bigint start with 1 increment by 50;

create table c_user (
  id                            numeric(19) not null,
  inactive                      integer default 0 not null,
  name                          nvarchar(255),
  email                         nvarchar(255),
  password_hash                 nvarchar(255),
  group_id                      numeric(19),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  constraint pk_c_user primary key (id)
);
create sequence c_user_seq as bigint start with 1 increment by 50;

create table tx_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_tx_user primary key (id)
);
create sequence tx_user_seq as bigint start with 1 increment by 50;

create table g_user (
  id                            numeric(19) not null,
  username                      nvarchar(255),
  version                       numeric(19) not null,
  constraint pk_g_user primary key (id)
);
create sequence g_user_seq as bigint start with 1 increment by 50;

create table em_user (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  constraint pk_em_user primary key (id)
);
create sequence em_user_seq as bigint start with 1 increment by 50;

create table user_interest_live (
  user_id                       numeric(19) not null,
  live_id                       numeric(19) not null,
  created_at                    datetime2 not null,
  constraint pk_user_interest_live primary key (user_id,live_id)
);

create table em_user_role (
  user_id                       numeric(19) not null,
  role_id                       numeric(19) not null,
  constraint pk_em_user_role primary key (user_id,role_id)
);

create table vehicle (
  dtype                         nvarchar(3) not null,
  id                            integer not null,
  license_number                nvarchar(255),
  registration_date             datetime2,
  lease_id                      numeric(19),
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  siz                           nvarchar(3),
  driver                        nvarchar(255),
  car_ref_id                    integer,
  notes                         nvarchar(255),
  truck_ref_id                  integer,
  capacity                      float(32),
  constraint ck_vehicle_siz check ( siz in ('S','M','L','H')),
  constraint pk_vehicle primary key (id)
);
create sequence vehicle_seq as bigint start with 1 increment by 50;

create table vehicle_driver (
  id                            integer not null,
  name                          nvarchar(255),
  vehicle_id                    integer,
  address_id                    integer,
  license_issued_on             datetime2,
  cretime                       datetime2 not null,
  updtime                       datetime2 not null,
  version                       numeric(19) not null,
  constraint pk_vehicle_driver primary key (id)
);
create sequence vehicle_driver_seq as bigint start with 1 increment by 50;

create table vehicle_lease (
  dtype                         nvarchar(31) not null,
  id                            numeric(19) not null,
  name                          nvarchar(255),
  active_start                  date,
  active_end                    date,
  version                       numeric(19) not null,
  bond                          numeric(16,3),
  min_duration                  integer not null,
  day_rate                      numeric(16,3),
  max_days                      integer,
  constraint pk_vehicle_lease primary key (id)
);
create sequence vehicle_lease_seq as bigint start with 1 increment by 50;

create table version_child (
  id                            integer not null,
  name                          nvarchar(255),
  parent_id                     integer,
  version                       integer not null,
  position                      integer,
  constraint pk_version_child primary key (id)
);
create sequence version_child_seq as bigint start with 1 increment by 50;

create table version_parent (
  id                            integer not null,
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_version_parent primary key (id)
);
create sequence version_parent_seq as bigint start with 1 increment by 50;

create table version_toy (
  id                            integer not null,
  name                          nvarchar(255),
  child_id                      integer,
  version                       integer not null,
  position                      integer,
  constraint pk_version_toy primary key (id)
);
create sequence version_toy_seq as bigint start with 1 increment by 50;

create table warehouses (
  id                            integer not null,
  officezoneid                  integer,
  constraint pk_warehouses primary key (id)
);
create sequence warehouses_seq as bigint start with 1 increment by 50;

create table warehousesshippingzones (
  warehouseid                   integer not null,
  shippingzoneid                integer not null,
  constraint pk_warehousesshippingzones primary key (warehouseid,shippingzoneid)
);

create table wheel (
  id                            numeric(19) not null,
  version                       integer not null,
  constraint pk_wheel primary key (id)
);
create sequence wheel_seq as bigint start with 1 increment by 50;

create table sa_wheel (
  id                            numeric(19) not null,
  tire                          numeric(19),
  car                           numeric(19),
  version                       integer not null,
  constraint pk_sa_wheel primary key (id)
);
create sequence sa_wheel_seq as bigint start with 1 increment by 50;

create table sp_car_wheel (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       integer not null,
  constraint pk_sp_car_wheel primary key (id)
);
create sequence sp_car_wheel_seq as bigint start with 1 increment by 50;

create table g_who_props_otm (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  version                       numeric(19) not null,
  when_created                  datetime2 not null,
  when_modified                 datetime2 not null,
  who_created_id                numeric(19),
  who_modified_id               numeric(19),
  constraint pk_g_who_props_otm primary key (id)
);
create sequence g_who_props_otm_seq as bigint start with 1 increment by 50;

create table with_zero (
  id                            numeric(19) not null,
  name                          nvarchar(255),
  parent_id                     integer,
  lang                          nvarchar(2) default 'en' not null,
  version                       numeric(19) not null,
  constraint pk_with_zero primary key (id)
);
create sequence with_zero_seq as bigint start with 1 increment by 50;

create table parent (
  id                            integer not null,
  name                          nvarchar(255),
  constraint pk_parent primary key (id)
);
create sequence parent_seq as bigint start with 1 increment by 50;

create table wview (
  id                            uniqueidentifier not null,
  name                          nvarchar(127) not null,
  constraint uq_wview_name unique (name),
  constraint pk_wview primary key (id)
);

create table zones (
  type                          nvarchar(31) not null,
  id                            integer not null,
  attribute                     nvarchar(255),
  constraint pk_zones primary key (id)
);
create sequence zones_seq as bigint start with 1 increment by 50;

create index ix_contact_last_name_first_name on contact (last_name,first_name);
create index ix_e_basic_name on e_basic (name);
create index ix_efile2_no_fk_owner_id on efile2_no_fk (owner_id);
create index ix_ecsm_values_host_id on ecsm_values (host_id);
create index ix_order_referenced_parent_type on order_referenced_parent (type);
create index ix_organization_node_kind on organization_node (kind);
create index ano_3 on test_annotation_base_entity (direct);
create index ix_bar_foo_id on bar (foo_id);
alter table bar add constraint fk_bar_foo_id foreign key (foo_id) references foo (foo_id);

create index ix_acl_container_relation_container_id on acl_container_relation (container_id);
alter table acl_container_relation add constraint fk_acl_container_relation_container_id foreign key (container_id) references contract (id);

create index ix_acl_container_relation_acl_entry_id on acl_container_relation (acl_entry_id);
alter table acl_container_relation add constraint fk_acl_container_relation_acl_entry_id foreign key (acl_entry_id) references acl (id);

create index ix_addr_employee_id on addr (employee_id);
alter table addr add constraint fk_addr_employee_id foreign key (employee_id) references empl (id);

create index ix_o_address_country_code on o_address (country_code);
alter table o_address add constraint fk_o_address_country_code foreign key (country_code) references o_country (code);

alter table album add constraint fk_album_cover_id foreign key (cover_id) references cover (id);

create index ix_animal_shelter_id on animal (shelter_id);
alter table animal add constraint fk_animal_shelter_id foreign key (shelter_id) references animal_shelter (id);

create index ix_app_config_control_config_id on app_config_control (config_id);
alter table app_config_control add constraint fk_app_config_control_config_id foreign key (config_id) references app_config (id);

create index ix_attribute_attribute_holder_id on attribute (attribute_holder_id);
alter table attribute add constraint fk_attribute_attribute_holder_id foreign key (attribute_holder_id) references attribute_holder (id);

create index ix_bbookmark_user_id on bbookmark (user_id);
alter table bbookmark add constraint fk_bbookmark_user_id foreign key (user_id) references bbookmark_user (id);

create index ix_bbookmark_user_org_id on bbookmark_user (org_id);
alter table bbookmark_user add constraint fk_bbookmark_user_org_id foreign key (org_id) references bbookmark_org (id);

create index ix_bsite_user_a_site_id on bsite_user_a (site_id);
alter table bsite_user_a add constraint fk_bsite_user_a_site_id foreign key (site_id) references bsite (id);

create index ix_bsite_user_a_user_id on bsite_user_a (user_id);
alter table bsite_user_a add constraint fk_bsite_user_a_user_id foreign key (user_id) references buser (id);

create index ix_bsite_user_b_site on bsite_user_b (site);
alter table bsite_user_b add constraint fk_bsite_user_b_site foreign key (site) references bsite (id);

create index ix_bsite_user_b_usr on bsite_user_b (usr);
alter table bsite_user_b add constraint fk_bsite_user_b_usr foreign key (usr) references buser (id);

create index ix_bsite_user_c_site_uid on bsite_user_c (site_uid);
alter table bsite_user_c add constraint fk_bsite_user_c_site_uid foreign key (site_uid) references bsite (id);

create index ix_bsite_user_c_user_uid on bsite_user_c (user_uid);
alter table bsite_user_c add constraint fk_bsite_user_c_user_uid foreign key (user_uid) references buser (id);

create index ix_bsite_user_dmany_bsite_user_d on bsite_user_dmany (site_id,user_id);
alter table bsite_user_dmany add constraint fk_bsite_user_dmany_bsite_user_d foreign key (site_id,user_id) references bsite_user_d (site_id,user_id);

create index ix_bsite_user_e_site_id on bsite_user_e (site_id);
alter table bsite_user_e add constraint fk_bsite_user_e_site_id foreign key (site_id) references bsite (id);

create index ix_bsite_user_e_user_id on bsite_user_e (user_id);
alter table bsite_user_e add constraint fk_bsite_user_e_user_id foreign key (user_id) references buser (id);

alter table basic_draftable_bean add constraint fk_basic_draftable_bean_id foreign key (id) references basic_draftable_bean_draft (id);

alter table drel_booking add constraint fk_drel_booking_agent_invoice foreign key (agent_invoice) references drel_invoice (id);

alter table drel_booking add constraint fk_drel_booking_client_invoice foreign key (client_invoice) references drel_invoice (id);

create index ix_cepproduct_category_category_id on cepproduct_category (category_id);
alter table cepproduct_category add constraint fk_cepproduct_category_category_id foreign key (category_id) references cepcategory (id);

create index ix_cepproduct_category_product_id on cepproduct_category (product_id);
alter table cepproduct_category add constraint fk_cepproduct_category_product_id foreign key (product_id) references cepproduct (id);

create index ix_ciaddress_street_id on ciaddress (street_id);
alter table ciaddress add constraint fk_ciaddress_street_id foreign key (street_id) references cistreet_parent (id);

create index ix_cicustomer_parent_address_id on cicustomer_parent (address_id);
alter table cicustomer_parent add constraint fk_cicustomer_parent_address_id foreign key (address_id) references ciaddress (id);

create index ix_cinh_ref_ref_id on cinh_ref (ref_id);
alter table cinh_ref add constraint fk_cinh_ref_ref_id foreign key (ref_id) references cinh_root (id);

create index ix_ckey_detail_parent on ckey_detail (one_key,two_key);
alter table ckey_detail add constraint fk_ckey_detail_parent foreign key (one_key,two_key) references ckey_parent (one_key,two_key);

create index ix_ckey_parent_assoc_id on ckey_parent (assoc_id);
alter table ckey_parent add constraint fk_ckey_parent_assoc_id foreign key (assoc_id) references ckey_assoc (id);

create index ix_coone_many_coone_id on coone_many (coone_id);
alter table coone_many add constraint fk_coone_many_coone_id foreign key (coone_id) references coone (id);

alter table coroot add constraint fk_coroot_one_id foreign key (one_id) references coone (id);

alter table calcd_data add constraint fk_calcd_data_id foreign key (id) references calcd_input (id) on delete cascade;

create index ix_calculation_result_product_configuration_id on calculation_result (product_configuration_id);
alter table calculation_result add constraint fk_calculation_result_product_configuration_id foreign key (product_configuration_id) references configuration (id);

create index ix_calculation_result_group_configuration_id on calculation_result (group_configuration_id);
alter table calculation_result add constraint fk_calculation_result_group_configuration_id foreign key (group_configuration_id) references configuration (id);

create index ix_sp_car_car_wheels_sp_car_car on sp_car_car_wheels (car);
alter table sp_car_car_wheels add constraint fk_sp_car_car_wheels_sp_car_car foreign key (car) references sp_car_car (id);

create index ix_sp_car_car_wheels_sp_car_wheel on sp_car_car_wheels (wheel);
alter table sp_car_car_wheels add constraint fk_sp_car_car_wheels_sp_car_wheel foreign key (wheel) references sp_car_wheel (id);

create index ix_sp_car_car_doors_sp_car_car on sp_car_car_doors (car);
alter table sp_car_car_doors add constraint fk_sp_car_car_doors_sp_car_car foreign key (car) references sp_car_car (id);

create index ix_sp_car_car_doors_sp_car_door on sp_car_car_doors (door);
alter table sp_car_car_doors add constraint fk_sp_car_car_doors_sp_car_door foreign key (door) references sp_car_door (id);

create index ix_car_accessory_fuse_id on car_accessory (fuse_id);
alter table car_accessory add constraint fk_car_accessory_fuse_id foreign key (fuse_id) references car_fuse (id);

create index ix_car_accessory_car_id on car_accessory (car_id);
alter table car_accessory add constraint fk_car_accessory_car_id foreign key (car_id) references vehicle (id);

create index ix_category_surveyobjectid on category (surveyobjectid);
alter table category add constraint fk_category_surveyobjectid foreign key (surveyobjectid) references survey (id);

alter table e_save_test_d add constraint fk_e_save_test_d_parent_id foreign key (parent_id) references e_save_test_c (id);

create index ix_child_person_some_bean_id on child_person (some_bean_id);
alter table child_person add constraint fk_child_person_some_bean_id foreign key (some_bean_id) references e_basic (id);

create index ix_child_person_parent_identifier on child_person (parent_identifier);
alter table child_person add constraint fk_child_person_parent_identifier foreign key (parent_identifier) references parent_person (identifier);

create index ix_cke_client_user on cke_client (username,cod_cpny);
alter table cke_client add constraint fk_cke_client_user foreign key (username,cod_cpny) references cke_user (username,cod_cpny);

alter table class_super_monkey add constraint fk_class_super_monkey_class_super foreign key (class_super_sid) references class_super (sid);

alter table class_super_monkey add constraint fk_class_super_monkey_monkey foreign key (monkey_mid) references monkey (mid);

create index ix_configuration_configurations_id on configuration (configurations_id);
alter table configuration add constraint fk_configuration_configurations_id foreign key (configurations_id) references configurations (id);

create index ix_contact_customer_id on contact (customer_id);
alter table contact add constraint fk_contact_customer_id foreign key (customer_id) references o_customer (id);

create index ix_contact_group_id on contact (group_id);
alter table contact add constraint fk_contact_group_id foreign key (group_id) references contact_group (id);

create index ix_contact_note_contact_id on contact_note (contact_id);
alter table contact_note add constraint fk_contact_note_contact_id foreign key (contact_id) references contact (id);

create index ix_contract_costs_position_id on contract_costs (position_id);
alter table contract_costs add constraint fk_contract_costs_position_id foreign key (position_id) references e_position (id);

create index ix_c_conversation_group_id on c_conversation (group_id);
alter table c_conversation add constraint fk_c_conversation_group_id foreign key (group_id) references c_group (id);

create index ix_o_customer_billing_address_id on o_customer (billing_address_id);
alter table o_customer add constraint fk_o_customer_billing_address_id foreign key (billing_address_id) references o_address (id);

create index ix_o_customer_shipping_address_id on o_customer (shipping_address_id);
alter table o_customer add constraint fk_o_customer_shipping_address_id foreign key (shipping_address_id) references o_address (id);

create index ix_dcredit_drol_dcredit on dcredit_drol (dcredit_id);
alter table dcredit_drol add constraint fk_dcredit_drol_dcredit foreign key (dcredit_id) references dcredit (id);

create index ix_dcredit_drol_drol on dcredit_drol (drol_id);
alter table dcredit_drol add constraint fk_dcredit_drol_drol foreign key (drol_id) references drol (id);

create index ix_dmachine_organisation_id on dmachine (organisation_id);
alter table dmachine add constraint fk_dmachine_organisation_id foreign key (organisation_id) references dorg (id);

create index ix_d_machine_aux_use_machine_id on d_machine_aux_use (machine_id);
alter table d_machine_aux_use add constraint fk_d_machine_aux_use_machine_id foreign key (machine_id) references dmachine (id);

create index ix_d_machine_stats_machine_id on d_machine_stats (machine_id);
alter table d_machine_stats add constraint fk_d_machine_stats_machine_id foreign key (machine_id) references dmachine (id);

create index ix_d_machine_use_machine_id on d_machine_use (machine_id);
alter table d_machine_use add constraint fk_d_machine_use_machine_id foreign key (machine_id) references dmachine (id);

create index ix_drot_drol_drot on drot_drol (drot_id);
alter table drot_drol add constraint fk_drot_drol_drot foreign key (drot_id) references drot (id);

create index ix_drot_drol_drol on drot_drol (drol_id);
alter table drot_drol add constraint fk_drot_drol_drol foreign key (drol_id) references drol (id);

create index ix_dc_detail_master_id on dc_detail (master_id);
alter table dc_detail add constraint fk_dc_detail_master_id foreign key (master_id) references dc_master (id);

alter table defaults_model add constraint fk_defaults_model_id foreign key (id) references defaults_model_draft (id);

create index ix_dfk_cascade_one_id on dfk_cascade (one_id);
alter table dfk_cascade add constraint fk_dfk_cascade_one_id foreign key (one_id) references dfk_cascade_one (id) on delete cascade;

create index ix_dfk_set_null_one_id on dfk_set_null (one_id);
alter table dfk_set_null add constraint fk_dfk_set_null_one_id foreign key (one_id) references dfk_one (id) on delete set null;

alter table doc add constraint fk_doc_id foreign key (id) references doc_draft (id);

create index ix_doc_link_doc on doc_link (doc_id);
alter table doc_link add constraint fk_doc_link_doc foreign key (doc_id) references doc (id);

create index ix_doc_link_link on doc_link (link_id);
alter table doc_link add constraint fk_doc_link_link foreign key (link_id) references link (id);

alter table document add constraint fk_document_id foreign key (id) references document_draft (id);

create index ix_document_organisation_id on document (organisation_id);
alter table document add constraint fk_document_organisation_id foreign key (organisation_id) references organisation (id);

create index ix_document_draft_organisation_id on document_draft (organisation_id);
alter table document_draft add constraint fk_document_draft_organisation_id foreign key (organisation_id) references organisation (id);

create index ix_document_media_document_id on document_media (document_id);
alter table document_media add constraint fk_document_media_document_id foreign key (document_id) references document (id);

create index ix_document_media_draft_document_id on document_media_draft (document_id);
alter table document_media_draft add constraint fk_document_media_draft_document_id foreign key (document_id) references document_draft (id);

create index ix_e_basicenc_relate_other_id on e_basicenc_relate (other_id);
alter table e_basicenc_relate add constraint fk_e_basicenc_relate_other_id foreign key (other_id) references e_basicenc (id);

create index ix_ebasic_json_map_detail_owner_id on ebasic_json_map_detail (owner_id);
alter table ebasic_json_map_detail add constraint fk_ebasic_json_map_detail_owner_id foreign key (owner_id) references ebasic_json_map (id);

create index ix_ebasic_no_sdchild_owner_id on ebasic_no_sdchild (owner_id);
alter table ebasic_no_sdchild add constraint fk_ebasic_no_sdchild_owner_id foreign key (owner_id) references ebasic_soft_delete (id);

create index ix_ebasic_sdchild_owner_id on ebasic_sdchild (owner_id);
alter table ebasic_sdchild add constraint fk_ebasic_sdchild_owner_id foreign key (owner_id) references ebasic_soft_delete (id);

create index ix_ecache_child_root_id on ecache_child (root_id);
alter table ecache_child add constraint fk_ecache_child_root_id foreign key (root_id) references ecache_root (id);

alter table edefault_prop add constraint fk_edefault_prop_e_simple_usertypeid foreign key (e_simple_usertypeid) references esimple (usertypeid);

create index ix_eemb_inner_outer_id on eemb_inner (outer_id);
alter table eemb_inner add constraint fk_eemb_inner_outer_id foreign key (outer_id) references eemb_outer (id);

create index ix_einvoice_person_id on einvoice (person_id);
alter table einvoice add constraint fk_einvoice_person_id foreign key (person_id) references eperson (id);

create index ix_enull_collection_detail_enull_collection_id on enull_collection_detail (enull_collection_id);
alter table enull_collection_detail add constraint fk_enull_collection_detail_enull_collection_id foreign key (enull_collection_id) references enull_collection (id);

create index ix_eopt_one_a_b_id on eopt_one_a (b_id);
alter table eopt_one_a add constraint fk_eopt_one_a_b_id foreign key (b_id) references eopt_one_b (id);

create index ix_eopt_one_b_c_id on eopt_one_b (c_id);
alter table eopt_one_b add constraint fk_eopt_one_b_c_id foreign key (c_id) references eopt_one_c (id);

create index ix_eper_addr_ma_country_code on eper_addr (ma_country_code);
alter table eper_addr add constraint fk_eper_addr_ma_country_code foreign key (ma_country_code) references o_country (code);

create index ix_esoft_del_book_lend_by_id on esoft_del_book (lend_by_id);
alter table esoft_del_book add constraint fk_esoft_del_book_lend_by_id foreign key (lend_by_id) references esoft_del_user (id);

create index ix_esoft_del_book_esoft_del_user_esoft_del_book on esoft_del_book_esoft_del_user (esoft_del_book_id);
alter table esoft_del_book_esoft_del_user add constraint fk_esoft_del_book_esoft_del_user_esoft_del_book foreign key (esoft_del_book_id) references esoft_del_book (id);

create index ix_esoft_del_book_esoft_del_user_esoft_del_user on esoft_del_book_esoft_del_user (esoft_del_user_id);
alter table esoft_del_book_esoft_del_user add constraint fk_esoft_del_book_esoft_del_user_esoft_del_user foreign key (esoft_del_user_id) references esoft_del_user (id);

create index ix_esoft_del_down_esoft_del_mid_id on esoft_del_down (esoft_del_mid_id);
alter table esoft_del_down add constraint fk_esoft_del_down_esoft_del_mid_id foreign key (esoft_del_mid_id) references esoft_del_mid (id);

create index ix_esoft_del_mid_top_id on esoft_del_mid (top_id);
alter table esoft_del_mid add constraint fk_esoft_del_mid_top_id foreign key (top_id) references esoft_del_top (id);

create index ix_esoft_del_mid_up_id on esoft_del_mid (up_id);
alter table esoft_del_mid add constraint fk_esoft_del_mid_up_id foreign key (up_id) references esoft_del_up (id);

alter table esoft_del_one_a add constraint fk_esoft_del_one_a_oneb_id foreign key (oneb_id) references esoft_del_one_b (id);

create index ix_esoft_del_role_esoft_del_user_esoft_del_role on esoft_del_role_esoft_del_user (esoft_del_role_id);
alter table esoft_del_role_esoft_del_user add constraint fk_esoft_del_role_esoft_del_user_esoft_del_role foreign key (esoft_del_role_id) references esoft_del_role (id);

create index ix_esoft_del_role_esoft_del_user_esoft_del_user on esoft_del_role_esoft_del_user (esoft_del_user_id);
alter table esoft_del_role_esoft_del_user add constraint fk_esoft_del_role_esoft_del_user_esoft_del_user foreign key (esoft_del_user_id) references esoft_del_user (id);

create index ix_esoft_del_user_esoft_del_role_esoft_del_user on esoft_del_user_esoft_del_role (esoft_del_user_id);
alter table esoft_del_user_esoft_del_role add constraint fk_esoft_del_user_esoft_del_role_esoft_del_user foreign key (esoft_del_user_id) references esoft_del_user (id);

create index ix_esoft_del_user_esoft_del_role_esoft_del_role on esoft_del_user_esoft_del_role (esoft_del_role_id);
alter table esoft_del_user_esoft_del_role add constraint fk_esoft_del_user_esoft_del_role_esoft_del_role foreign key (esoft_del_role_id) references esoft_del_role (id);

alter table esoft_del_x add constraint fk_esoft_del_x_y_id foreign key (y_id) references esoft_del_y (id);

create index ix_esoft_del_x_organization_id on esoft_del_x (organization_id);
alter table esoft_del_x add constraint fk_esoft_del_x_organization_id foreign key (organization_id) references esoft_del_z (id);

create index ix_esoft_del_y_organization_id on esoft_del_y (organization_id);
alter table esoft_del_y add constraint fk_esoft_del_y_organization_id foreign key (organization_id) references esoft_del_z (id);

create index ix_rawinherit_uncle_parent_id on rawinherit_uncle (parent_id);
alter table rawinherit_uncle add constraint fk_rawinherit_uncle_parent_id foreign key (parent_id) references rawinherit_parent (id);

create index ix_evanilla_collection_detail_evanilla_collection_id on evanilla_collection_detail (evanilla_collection_id);
alter table evanilla_collection_detail add constraint fk_evanilla_collection_detail_evanilla_collection_id foreign key (evanilla_collection_id) references evanilla_collection (id);

create index ix_ec_enum_person_tags_ec_enum_person_id on ec_enum_person_tags (ec_enum_person_id);
alter table ec_enum_person_tags add constraint fk_ec_enum_person_tags_ec_enum_person_id foreign key (ec_enum_person_id) references ec_enum_person (id);

create index ix_ec_person_phone_owner_id on ec_person_phone (owner_id);
alter table ec_person_phone add constraint fk_ec_person_phone_owner_id foreign key (owner_id) references ec_person (id);

create index ix_ec_top_person_id on ec_top (person_id);
alter table ec_top add constraint fk_ec_top_person_id foreign key (person_id) references ecs_person (id);

create index ix_ec_top_ecs_person_ec_top on ec_top_ecs_person (ec_top_id);
alter table ec_top_ecs_person add constraint fk_ec_top_ecs_person_ec_top foreign key (ec_top_id) references ec_top (id);

create index ix_ec_top_ecs_person_ecs_person on ec_top_ecs_person (ecs_person_id);
alter table ec_top_ecs_person add constraint fk_ec_top_ecs_person_ecs_person foreign key (ecs_person_id) references ecs_person (id);

create index ix_ecbl_person_phone_numbers_person_id on ecbl_person_phone_numbers (person_id);
alter table ecbl_person_phone_numbers add constraint fk_ecbl_person_phone_numbers_person_id foreign key (person_id) references ecbl_person (id);

create index ix_ecbm_person_phone_numbers_person_id on ecbm_person_phone_numbers (person_id);
alter table ecbm_person_phone_numbers add constraint fk_ecbm_person_phone_numbers_person_id foreign key (person_id) references ecbm_person (id);

create index ix_ecm_person_phone_numbers_ecm_person_id on ecm_person_phone_numbers (ecm_person_id);
alter table ecm_person_phone_numbers add constraint fk_ecm_person_phone_numbers_ecm_person_id foreign key (ecm_person_id) references ecm_person (id);

create index ix_ecmc_person_phone_numbers_ecmc_person_id on ecmc_person_phone_numbers (ecmc_person_id);
alter table ecmc_person_phone_numbers add constraint fk_ecmc_person_phone_numbers_ecmc_person_id foreign key (ecmc_person_id) references ecmc_person (id);

create index ix_ecs_person_phone_ecs_person_id on ecs_person_phone (ecs_person_id);
alter table ecs_person_phone add constraint fk_ecs_person_phone_ecs_person_id foreign key (ecs_person_id) references ecs_person (id);

create index ix_ecsm_child_ecsm_parent_id on ecsm_child (ecsm_parent_id);
alter table ecsm_child add constraint fk_ecsm_child_ecsm_parent_id foreign key (ecsm_parent_id) references ecsm_parent (id);

create index ix_td_child_parent_id on td_child (parent_id);
alter table td_child add constraint fk_td_child_parent_id foreign key (parent_id) references td_parent (parent_id);

create index ix_element_bean_complex_bean_id on element_bean (complex_bean_id);
alter table element_bean add constraint fk_element_bean_complex_bean_id foreign key (complex_bean_id) references root_bean (id);

create index ix_empl_default_address_id on empl (default_address_id);
alter table empl add constraint fk_empl_default_address_id foreign key (default_address_id) references addr (id);

create index ix_esd_detail_master_id on esd_detail (master_id);
alter table esd_detail add constraint fk_esd_detail_master_id foreign key (master_id) references esd_master (id);

create index ix_grand_parent_person_some_bean_id on grand_parent_person (some_bean_id);
alter table grand_parent_person add constraint fk_grand_parent_person_some_bean_id foreign key (some_bean_id) references e_basic (id);

create index ix_survey_group_categoryobjectid on survey_group (categoryobjectid);
alter table survey_group add constraint fk_survey_group_categoryobjectid foreign key (categoryobjectid) references category (id);

create index ix_hx_link_doc_hx_link on hx_link_doc (hx_link_id);
alter table hx_link_doc add constraint fk_hx_link_doc_hx_link foreign key (hx_link_id) references hx_link (id);

create index ix_hx_link_doc_he_doc on hx_link_doc (he_doc_id);
alter table hx_link_doc add constraint fk_hx_link_doc_he_doc foreign key (he_doc_id) references he_doc (id);

create index ix_hi_link_doc_hi_link on hi_link_doc (hi_link_id);
alter table hi_link_doc add constraint fk_hi_link_doc_hi_link foreign key (hi_link_id) references hi_link (id);

create index ix_hi_link_doc_hi_doc on hi_link_doc (hi_doc_id);
alter table hi_link_doc add constraint fk_hi_link_doc_hi_doc foreign key (hi_doc_id) references hi_doc (id);

create index ix_hi_tthree_hi_ttwo_id on hi_tthree (hi_ttwo_id);
alter table hi_tthree add constraint fk_hi_tthree_hi_ttwo_id foreign key (hi_ttwo_id) references hi_ttwo (id);

create index ix_hi_ttwo_hi_tone_id on hi_ttwo (hi_tone_id);
alter table hi_ttwo add constraint fk_hi_ttwo_hi_tone_id foreign key (hi_tone_id) references hi_tone (id);

alter table hsd_setting add constraint fk_hsd_setting_user_id foreign key (user_id) references hsd_user (id);

create index ix_iaf_segment_status_id on iaf_segment (status_id);
alter table iaf_segment add constraint fk_iaf_segment_status_id foreign key (status_id) references iaf_segment_status (id);

create index ix_imrelated_owner_id on imrelated (owner_id);
alter table imrelated add constraint fk_imrelated_owner_id foreign key (owner_id) references imroot (id);

create index ix_info_contact_company_id on info_contact (company_id);
alter table info_contact add constraint fk_info_contact_company_id foreign key (company_id) references info_company (id);

alter table info_customer add constraint fk_info_customer_company_id foreign key (company_id) references info_company (id);

alter table inner_report add constraint fk_inner_report_forecast_id foreign key (forecast_id) references stockforecast (id);

create index ix_drel_invoice_booking on drel_invoice (booking);
alter table drel_invoice add constraint fk_drel_invoice_booking foreign key (booking) references drel_booking (id);

create index ix_item_etype on item (customer,type);
alter table item add constraint fk_item_etype foreign key (customer,type) references [type] (customer,type);

create index ix_item_eregion on item (customer,region);
alter table item add constraint fk_item_eregion foreign key (customer,region) references region (customer,type);

alter table mkeygroup_monkey add constraint fk_mkeygroup_monkey_mkeygroup foreign key (mkeygroup_pid) references mkeygroup (pid);

alter table mkeygroup_monkey add constraint fk_mkeygroup_monkey_monkey foreign key (monkey_mid) references monkey (mid);

alter table trainer_monkey add constraint fk_trainer_monkey_trainer foreign key (trainer_tid) references trainer (tid);

alter table trainer_monkey add constraint fk_trainer_monkey_monkey foreign key (monkey_mid) references monkey (mid);

alter table troop_monkey add constraint fk_troop_monkey_troop foreign key (troop_pid) references troop (pid);

alter table troop_monkey add constraint fk_troop_monkey_monkey foreign key (monkey_mid) references monkey (mid);

create index ix_l2_cldf_reset_bean_child_parent_id on l2_cldf_reset_bean_child (parent_id);
alter table l2_cldf_reset_bean_child add constraint fk_l2_cldf_reset_bean_child_parent_id foreign key (parent_id) references l2_cldf_reset_bean (id);

create index ix_level1_level4_level1 on level1_level4 (level1_id);
alter table level1_level4 add constraint fk_level1_level4_level1 foreign key (level1_id) references level1 (id);

create index ix_level1_level4_level4 on level1_level4 (level4_id);
alter table level1_level4 add constraint fk_level1_level4_level4 foreign key (level4_id) references level4 (id);

create index ix_level1_level2_level1 on level1_level2 (level1_id);
alter table level1_level2 add constraint fk_level1_level2_level1 foreign key (level1_id) references level1 (id);

create index ix_level1_level2_level2 on level1_level2 (level2_id);
alter table level1_level2 add constraint fk_level1_level2_level2 foreign key (level2_id) references level2 (id);

create index ix_level2_level3_level2 on level2_level3 (level2_id);
alter table level2_level3 add constraint fk_level2_level3_level2 foreign key (level2_id) references level2 (id);

create index ix_level2_level3_level3 on level2_level3 (level3_id);
alter table level2_level3 add constraint fk_level2_level3_level3 foreign key (level3_id) references level3 (id);

alter table link add constraint fk_link_id foreign key (id) references link_draft (id);

create index ix_la_attr_value_attribute_la_attr_value on la_attr_value_attribute (la_attr_value_id);
alter table la_attr_value_attribute add constraint fk_la_attr_value_attribute_la_attr_value foreign key (la_attr_value_id) references la_attr_value (id);

create index ix_la_attr_value_attribute_attribute on la_attr_value_attribute (attribute_id);
alter table la_attr_value_attribute add constraint fk_la_attr_value_attribute_attribute foreign key (attribute_id) references attribute (id);

create index ix_looney_tune_id on looney (tune_id);
alter table looney add constraint fk_looney_tune_id foreign key (tune_id) references tune (id);

create index ix_m2mcache_set1_m2_mcache_master on m2mcache_set1 (m2_mcache_master_id);
alter table m2mcache_set1 add constraint fk_m2mcache_set1_m2_mcache_master foreign key (m2_mcache_master_id) references m2_mcache_master (id);

create index ix_m2mcache_set1_m2_mcache_child on m2mcache_set1 (m2_mcache_child_id);
alter table m2mcache_set1 add constraint fk_m2mcache_set1_m2_mcache_child foreign key (m2_mcache_child_id) references m2_mcache_child (id);

create index ix_m2mcache_set2_m2_mcache_master on m2mcache_set2 (m2_mcache_master_id);
alter table m2mcache_set2 add constraint fk_m2mcache_set2_m2_mcache_master foreign key (m2_mcache_master_id) references m2_mcache_master (id);

create index ix_m2mcache_set2_m2_mcache_child on m2mcache_set2 (m2_mcache_child_id);
alter table m2mcache_set2 add constraint fk_m2mcache_set2_m2_mcache_child foreign key (m2_mcache_child_id) references m2_mcache_child (id);

create index ix_mcontact_customer_id on mcontact (customer_id);
alter table mcontact add constraint fk_mcontact_customer_id foreign key (customer_id) references mcustomer (id);

create index ix_mcontact_message_contact_id on mcontact_message (contact_id);
alter table mcontact_message add constraint fk_mcontact_message_contact_id foreign key (contact_id) references mcontact (id);

create index ix_mcustomer_shipping_address_id on mcustomer (shipping_address_id);
alter table mcustomer add constraint fk_mcustomer_shipping_address_id foreign key (shipping_address_id) references maddress (id);

create index ix_mcustomer_billing_address_id on mcustomer (billing_address_id);
alter table mcustomer add constraint fk_mcustomer_billing_address_id foreign key (billing_address_id) references maddress (id);

create index ix_mmachine_mgroup_mmachine on mmachine_mgroup (mmachine_id);
alter table mmachine_mgroup add constraint fk_mmachine_mgroup_mmachine foreign key (mmachine_id) references mmachine (id);

create index ix_mmachine_mgroup_mgroup on mmachine_mgroup (mgroup_id);
alter table mmachine_mgroup add constraint fk_mmachine_mgroup_mgroup foreign key (mgroup_id) references mgroup (id);

create index ix_mprinter_current_state_id on mprinter (current_state_id);
alter table mprinter add constraint fk_mprinter_current_state_id foreign key (current_state_id) references mprinter_state (id);

alter table mprinter add constraint fk_mprinter_last_swap_cyan_id foreign key (last_swap_cyan_id) references mprinter_state (id);

alter table mprinter add constraint fk_mprinter_last_swap_magenta_id foreign key (last_swap_magenta_id) references mprinter_state (id);

alter table mprinter add constraint fk_mprinter_last_swap_yellow_id foreign key (last_swap_yellow_id) references mprinter_state (id);

alter table mprinter add constraint fk_mprinter_last_swap_black_id foreign key (last_swap_black_id) references mprinter_state (id);

create index ix_mprinter_state_printer_id on mprinter_state (printer_id);
alter table mprinter_state add constraint fk_mprinter_state_printer_id foreign key (printer_id) references mprinter (id);

create index ix_mprofile_picture_id on mprofile (picture_id);
alter table mprofile add constraint fk_mprofile_picture_id foreign key (picture_id) references mmedia (id);

create index ix_mrole_muser_mrole on mrole_muser (mrole_roleid);
alter table mrole_muser add constraint fk_mrole_muser_mrole foreign key (mrole_roleid) references mrole (roleid);

create index ix_mrole_muser_muser on mrole_muser (muser_userid);
alter table mrole_muser add constraint fk_mrole_muser_muser foreign key (muser_userid) references muser (userid);

create index ix_muser_user_type_id on muser (user_type_id);
alter table muser add constraint fk_muser_user_type_id foreign key (user_type_id) references muser_type (id);

create index ix_mail_user_inbox_mail_user on mail_user_inbox (mail_user_id);
alter table mail_user_inbox add constraint fk_mail_user_inbox_mail_user foreign key (mail_user_id) references mail_user (id);

create index ix_mail_user_inbox_mail_box on mail_user_inbox (mail_box_id);
alter table mail_user_inbox add constraint fk_mail_user_inbox_mail_box foreign key (mail_box_id) references mail_box (id);

create index ix_mail_user_outbox_mail_user on mail_user_outbox (mail_user_id);
alter table mail_user_outbox add constraint fk_mail_user_outbox_mail_user foreign key (mail_user_id) references mail_user (id);

create index ix_mail_user_outbox_mail_box on mail_user_outbox (mail_box_id);
alter table mail_user_outbox add constraint fk_mail_user_outbox_mail_box foreign key (mail_box_id) references mail_box (id);

create index ix_c_message_conversation_id on c_message (conversation_id);
alter table c_message add constraint fk_c_message_conversation_id foreign key (conversation_id) references c_conversation (id);

create index ix_c_message_user_id on c_message (user_id);
alter table c_message add constraint fk_c_message_user_id foreign key (user_id) references c_user (id);

alter table meter_contract_data add constraint fk_meter_contract_data_special_needs_client_id foreign key (special_needs_client_id) references meter_special_needs_client (id);

alter table meter_special_needs_client add constraint fk_meter_special_needs_client_primary_id foreign key (primary_id) references meter_special_needs_contact (id);

alter table meter_version add constraint fk_meter_version_address_data_id foreign key (address_data_id) references meter_address_data (id);

alter table meter_version add constraint fk_meter_version_contract_data_id foreign key (contract_data_id) references meter_contract_data (id);

create index ix_mnoc_user_mnoc_role_mnoc_user on mnoc_user_mnoc_role (mnoc_user_user_id);
alter table mnoc_user_mnoc_role add constraint fk_mnoc_user_mnoc_role_mnoc_user foreign key (mnoc_user_user_id) references mnoc_user (user_id);

create index ix_mnoc_user_mnoc_role_mnoc_role on mnoc_user_mnoc_role (mnoc_role_role_id);
alter table mnoc_user_mnoc_role add constraint fk_mnoc_user_mnoc_role_mnoc_role foreign key (mnoc_role_role_id) references mnoc_role (role_id);

create index ix_mny_b_a_id on mny_b (a_id);
alter table mny_b add constraint fk_mny_b_a_id foreign key (a_id) references mny_a (id);

create index ix_mny_b_mny_c_mny_b on mny_b_mny_c (mny_b_id);
alter table mny_b_mny_c add constraint fk_mny_b_mny_c_mny_b foreign key (mny_b_id) references mny_b (id);

create index ix_mny_b_mny_c_mny_c on mny_b_mny_c (mny_c_id);
alter table mny_b_mny_c add constraint fk_mny_b_mny_c_mny_c foreign key (mny_c_id) references mny_c (id);

create index ix_mny_edge_from_id on mny_edge (from_id);
alter table mny_edge add constraint fk_mny_edge_from_id foreign key (from_id) references mny_node (id);

create index ix_mny_edge_to_id on mny_edge (to_id);
alter table mny_edge add constraint fk_mny_edge_to_id foreign key (to_id) references mny_node (id);

create index ix_subtopics_mny_topic_1 on subtopics (topic);
alter table subtopics add constraint fk_subtopics_mny_topic_1 foreign key (topic) references mny_topic (id);

create index ix_subtopics_mny_topic_2 on subtopics (subtopic);
alter table subtopics add constraint fk_subtopics_mny_topic_2 foreign key (subtopic) references mny_topic (id);

create index ix_mp_role_mp_user_id on mp_role (mp_user_id);
alter table mp_role add constraint fk_mp_role_mp_user_id foreign key (mp_user_id) references mp_user (id);

create index ix_ms_many_a_many_b_ms_many_a on ms_many_a_many_b (ms_many_a_aid);
alter table ms_many_a_many_b add constraint fk_ms_many_a_many_b_ms_many_a foreign key (ms_many_a_aid) references ms_many_a (aid);

create index ix_ms_many_a_many_b_ms_many_b on ms_many_a_many_b (ms_many_b_bid);
alter table ms_many_a_many_b add constraint fk_ms_many_a_many_b_ms_many_b foreign key (ms_many_b_bid) references ms_many_b (bid);

create index ix_ms_many_b_many_a_ms_many_b on ms_many_b_many_a (ms_many_b_bid);
alter table ms_many_b_many_a add constraint fk_ms_many_b_many_a_ms_many_b foreign key (ms_many_b_bid) references ms_many_b (bid);

create index ix_ms_many_b_many_a_ms_many_a on ms_many_b_many_a (ms_many_a_aid);
alter table ms_many_b_many_a add constraint fk_ms_many_b_many_a_ms_many_a foreign key (ms_many_a_aid) references ms_many_a (aid);

create index ix_my_lob_size_join_many_parent_id on my_lob_size_join_many (parent_id);
alter table my_lob_size_join_many add constraint fk_my_lob_size_join_many_parent_id foreign key (parent_id) references my_lob_size (id);

create index ix_o_bean_child_cached_bean_id on o_bean_child (cached_bean_id);
alter table o_bean_child add constraint fk_o_bean_child_cached_bean_id foreign key (cached_bean_id) references o_cached_bean (id);

create index ix_ocached_app_detail_app_id on ocached_app_detail (app_id);
alter table ocached_app_detail add constraint fk_ocached_app_detail_app_id foreign key (app_id) references ocached_app (id);

create index ix_o_cached_bean_country_o_cached_bean on o_cached_bean_country (o_cached_bean_id);
alter table o_cached_bean_country add constraint fk_o_cached_bean_country_o_cached_bean foreign key (o_cached_bean_id) references o_cached_bean (id);

create index ix_o_cached_bean_country_o_country on o_cached_bean_country (o_country_code);
alter table o_cached_bean_country add constraint fk_o_cached_bean_country_o_country foreign key (o_country_code) references o_country (code);

create index ix_o_cached_bean_child_cached_bean_id on o_cached_bean_child (cached_bean_id);
alter table o_cached_bean_child add constraint fk_o_cached_bean_child_cached_bean_id foreign key (cached_bean_id) references o_cached_bean (id);

alter table oengine add constraint fk_oengine_car_id foreign key (car_id) references ocar (id);

alter table ogear_box add constraint fk_ogear_box_car_id foreign key (car_id) references ocar (id);

create index ix_omvertex_other_omvertex_id on omvertex_other (omvertex_id);
alter table omvertex_other add constraint fk_omvertex_other_omvertex_id foreign key (omvertex_id) references omvertex (id);

alter table oroad_show_msg add constraint fk_oroad_show_msg_company_id foreign key (company_id) references ocompany (id);

create index ix_om_account_child_dbo_banana_rama_id on om_account_child_dbo (banana_rama_id);
alter table om_account_child_dbo add constraint fk_om_account_child_dbo_banana_rama_id foreign key (banana_rama_id) references om_account_dbo (id);

create index ix_om_basic_child_parent_id on om_basic_child (parent_id);
alter table om_basic_child add constraint fk_om_basic_child_parent_id foreign key (parent_id) references om_basic_parent (id);

create index ix_om_bean_list_child_parent_id on om_bean_list_child (parent_id);
alter table om_bean_list_child add constraint fk_om_bean_list_child_parent_id foreign key (parent_id) references om_bean_list_parent (id);

create index ix_om_ordered_detail_master_id on om_ordered_detail (master_id);
alter table om_ordered_detail add constraint fk_om_ordered_detail_master_id foreign key (master_id) references om_ordered_master (id);

create index ix_oml_baz_foo_id on oml_baz (foo_id);
alter table oml_baz add constraint fk_oml_baz_foo_id foreign key (foo_id) references oml_foo (id);

create index ix_oml_foo_bar_id on oml_foo (bar_id);
alter table oml_foo add constraint fk_oml_foo_bar_id foreign key (bar_id) references oml_bar (id);

create index ix_o_order_kcustomer_id on o_order (kcustomer_id);
alter table o_order add constraint fk_o_order_kcustomer_id foreign key (kcustomer_id) references o_customer (id);

create index ix_o_order_detail_order_id on o_order_detail (order_id);
alter table o_order_detail add constraint fk_o_order_detail_order_id foreign key (order_id) references o_order (id);

create index ix_o_order_detail_product_id on o_order_detail (product_id);
alter table o_order_detail add constraint fk_o_order_detail_product_id foreign key (product_id) references o_product (id);

create index ix_s_order_items_order_uuid on s_order_items (order_uuid);
alter table s_order_items add constraint fk_s_order_items_order_uuid foreign key (order_uuid) references s_orders (uuid);

create index ix_order_referenced_parent_master_id on order_referenced_parent (master_id);
alter table order_referenced_parent add constraint fk_order_referenced_parent_master_id foreign key (master_id) references order_master (id);

create index ix_or_order_ship_order_id on or_order_ship (order_id);
alter table or_order_ship add constraint fk_or_order_ship_order_id foreign key (order_id) references o_order (id);

create index ix_order_toy_child_id on order_toy (child_id);
alter table order_toy add constraint fk_order_toy_child_id foreign key (child_id) references order_referenced_parent (id);

create index ix_ordered_parent_order_master_inheritance_id on ordered_parent (order_master_inheritance_id);
alter table ordered_parent add constraint fk_ordered_parent_order_master_inheritance_id foreign key (order_master_inheritance_id) references order_master_inheritance (id);

alter table organization_node add constraint fk_organization_node_parent_tree_node_id foreign key (parent_tree_node_id) references organization_tree_node (id);

create index ix_orp_detail_master_id on orp_detail (master_id);
alter table orp_detail add constraint fk_orp_detail_master_id foreign key (master_id) references orp_master (id);

create index ix_orp_detail2_orp_master2_id on orp_detail2 (orp_master2_id);
alter table orp_detail2 add constraint fk_orp_detail2_orp_master2_id foreign key (orp_master2_id) references orp_master2 (id);

alter table oto_atwo add constraint fk_oto_atwo_aone_id foreign key (aone_id) references oto_aone (id);

alter table oto_bchild add constraint fk_oto_bchild_master_id foreign key (master_id) references oto_bmaster (id);

alter table oto_child add constraint fk_oto_child_master_id foreign key (master_id) references oto_master (id);

alter table oto_cust_address add constraint fk_oto_cust_address_customer_cid foreign key (customer_cid) references oto_cust (cid);

alter table oto_level_a add constraint fk_oto_level_a_b_id foreign key (b_id) references oto_level_b (id);

alter table oto_level_b add constraint fk_oto_level_b_c_id foreign key (c_id) references oto_level_c (id);

alter table oto_prime_extra add constraint fk_oto_prime_extra_eid foreign key (eid) references oto_prime (pid);

alter table oto_sd_child add constraint fk_oto_sd_child_master_id foreign key (master_id) references oto_sd_master (id);

create index ix_oto_th_many_oto_th_top_id on oto_th_many (oto_th_top_id);
alter table oto_th_many add constraint fk_oto_th_many_oto_th_top_id foreign key (oto_th_top_id) references oto_th_top (id);

alter table oto_th_one add constraint fk_oto_th_one_many_id foreign key (many_id) references oto_th_many (id);

alter table oto_ubprime_extra add constraint fk_oto_ubprime_extra_eid foreign key (eid) references oto_ubprime (pid);

alter table oto_user_model add constraint fk_oto_user_model_user_optional_id foreign key (user_optional_id) references oto_user_model_optional (id);

alter table pfile add constraint fk_pfile_file_content_id foreign key (file_content_id) references pfile_content (id);

alter table pfile add constraint fk_pfile_file_content2_id foreign key (file_content2_id) references pfile_content (id);

alter table paggview add constraint fk_paggview_pview_id foreign key (pview_id) references pp (id);

create index ix_pallet_location_zone_sid on pallet_location (zone_sid);
alter table pallet_location add constraint fk_pallet_location_zone_sid foreign key (zone_sid) references zones (id);

alter table parcel_location add constraint fk_parcel_location_parcelid foreign key (parcelid) references parcel (parcelid);

create index ix_rawinherit_parent_rawinherit_data_rawinherit_parent on rawinherit_parent_rawinherit_data (rawinherit_parent_id);
alter table rawinherit_parent_rawinherit_data add constraint fk_rawinherit_parent_rawinherit_data_rawinherit_parent foreign key (rawinherit_parent_id) references rawinherit_parent (id);

create index ix_rawinherit_parent_rawinherit_data_rawinherit_data on rawinherit_parent_rawinherit_data (rawinherit_data_id);
alter table rawinherit_parent_rawinherit_data add constraint fk_rawinherit_parent_rawinherit_data_rawinherit_data foreign key (rawinherit_data_id) references rawinherit_data (id);

create index ix_parent_a_child_id on parent_a (child_id);
alter table parent_a add constraint fk_parent_a_child_id foreign key (child_id) references child_with_cache (id);

create index ix_parent_b_child_id on parent_b (child_id);
alter table parent_b add constraint fk_parent_b_child_id foreign key (child_id) references child_with_cache (id);

create index ix_parent_person_some_bean_id on parent_person (some_bean_id);
alter table parent_person add constraint fk_parent_person_some_bean_id foreign key (some_bean_id) references e_basic (id);

create index ix_parent_person_parent_identifier on parent_person (parent_identifier);
alter table parent_person add constraint fk_parent_person_parent_identifier foreign key (parent_identifier) references grand_parent_person (identifier);

create index ix_c_participation_conversation_id on c_participation (conversation_id);
alter table c_participation add constraint fk_c_participation_conversation_id foreign key (conversation_id) references c_conversation (id);

create index ix_c_participation_user_id on c_participation (user_id);
alter table c_participation add constraint fk_c_participation_user_id foreign key (user_id) references c_user (id);

create index ix_pcf_calendar_pcf_person_id on pcf_calendar (pcf_person_id);
alter table pcf_calendar add constraint fk_pcf_calendar_pcf_person_id foreign key (pcf_person_id) references pcf_person (id);

create index ix_pcf_city_pcf_country_id on pcf_city (pcf_country_id);
alter table pcf_city add constraint fk_pcf_city_pcf_country_id foreign key (pcf_country_id) references pcf_country (id);

alter table pcf_city add constraint fk_pcf_city_mayor_id foreign key (mayor_id) references pcf_person (id);

alter table pcf_city add constraint fk_pcf_city_vice_mayor_id foreign key (vice_mayor_id) references pcf_person (id);

create index ix_pcf_event_pcf_calendar_id on pcf_event (pcf_calendar_id);
alter table pcf_event add constraint fk_pcf_event_pcf_calendar_id foreign key (pcf_calendar_id) references pcf_calendar (id);

alter table persistent_file_content add constraint fk_persistent_file_content_persistent_file_id foreign key (persistent_file_id) references persistent_file (id);

create index ix_person_default_address_oid on person (default_address_oid);
alter table person add constraint fk_person_default_address_oid foreign key (default_address_oid) references address (oid);

create index ix_person_cache_email_person_info_person_id on person_cache_email (person_info_person_id);
alter table person_cache_email add constraint fk_person_cache_email_person_info_person_id foreign key (person_info_person_id) references person_cache_info (person_id);

alter table persona add constraint fk_persona_person_oid foreign key (person_oid) references person (oid);

create index ix_phones_person_id on phones (person_id);
alter table phones add constraint fk_phones_person_id foreign key (person_id) references persons (id);

create index ix_e_position_contract_id on e_position (contract_id);
alter table e_position add constraint fk_e_position_contract_id foreign key (contract_id) references contract (id);

create index ix_pp_to_ww_pp on pp_to_ww (pp_id);
alter table pp_to_ww add constraint fk_pp_to_ww_pp foreign key (pp_id) references pp (id);

create index ix_pp_to_ww_wview on pp_to_ww (ww_id);
alter table pp_to_ww add constraint fk_pp_to_ww_wview foreign key (ww_id) references wview (id);

create index ix_question_groupobjectid on question (groupobjectid);
alter table question add constraint fk_question_groupobjectid foreign key (groupobjectid) references survey_group (id);

create index ix_r_orders_customer on r_orders (company,customername);
alter table r_orders add constraint fk_r_orders_customer foreign key (company,customername) references rcustomer (company,name);

alter table referenced_defaults_model add constraint fk_referenced_defaults_model_id foreign key (id) references referenced_defaults_model_draft (id);

create index ix_referenced_defaults_model_defaults_model_id on referenced_defaults_model (defaults_model_id);
alter table referenced_defaults_model add constraint fk_referenced_defaults_model_defaults_model_id foreign key (defaults_model_id) references defaults_model (id);

create index ix_referenced_defaults_model_draft_defaults_model_id on referenced_defaults_model_draft (defaults_model_id);
alter table referenced_defaults_model_draft add constraint fk_referenced_defaults_model_draft_defaults_model_id foreign key (defaults_model_id) references defaults_model_draft (id);

create index ix_rel_master_detail_id on rel_master (detail_id);
alter table rel_master add constraint fk_rel_master_detail_id foreign key (detail_id) references rel_detail (id);

create index ix_app_config_re_config_id on app_config_re (config_id);
alter table app_config_re add constraint fk_app_config_re_config_id foreign key (config_id) references app_config (id);

create index ix_resourcefile_parentresourcefileid on resourcefile (parentresourcefileid);
alter table resourcefile add constraint fk_resourcefile_parentresourcefileid foreign key (parentresourcefileid) references resourcefile (id);

create index ix_mt_role_tenant_id on mt_role (tenant_id);
alter table mt_role add constraint fk_mt_role_tenant_id foreign key (tenant_id) references mt_tenant (id);

create index ix_mt_role_permission_mt_role on mt_role_permission (mt_role_id);
alter table mt_role_permission add constraint fk_mt_role_permission_mt_role foreign key (mt_role_id) references mt_role (id);

create index ix_mt_role_permission_mt_permission on mt_role_permission (mt_permission_id);
alter table mt_role_permission add constraint fk_mt_role_permission_mt_permission foreign key (mt_permission_id) references mt_permission (id);

create index ix_root_bean_referencing_bean_id on root_bean (referencing_bean_id);
alter table root_bean add constraint fk_root_bean_referencing_bean_id foreign key (referencing_bean_id) references referencing_bean (id);

alter table f_second add constraint fk_f_second_first foreign key (first) references f_first (id);

create index ix_section_article_id on section (article_id);
alter table section add constraint fk_section_article_id foreign key (article_id) references article (id);

create index ix_self_many_bridge_self_many_many_1 on self_many_bridge (self_1_id);
alter table self_many_bridge add constraint fk_self_many_bridge_self_many_many_1 foreign key (self_1_id) references self_many_many (id);

create index ix_self_many_bridge_self_many_many_2 on self_many_bridge (self_2_id);
alter table self_many_bridge add constraint fk_self_many_bridge_self_many_many_2 foreign key (self_2_id) references self_many_many (id);

create index ix_self_parent_parent_id on self_parent (parent_id);
alter table self_parent add constraint fk_self_parent_parent_id foreign key (parent_id) references self_parent (id);

create index ix_self_ref_customer_referred_by_id on self_ref_customer (referred_by_id);
alter table self_ref_customer add constraint fk_self_ref_customer_referred_by_id foreign key (referred_by_id) references self_ref_customer (id);

create index ix_self_ref_example_parent_id on self_ref_example (parent_id);
alter table self_ref_example add constraint fk_self_ref_example_parent_id foreign key (parent_id) references self_ref_example (id);

alter table e_save_test_b add constraint fk_e_save_test_b_sibling_a_id foreign key (sibling_a_id) references e_save_test_a (id);

create index ix_site_parent_id on site (parent_id);
alter table site add constraint fk_site_parent_id foreign key (parent_id) references site (id);

alter table site add constraint fk_site_data_container_id foreign key (data_container_id) references data_container (id);

alter table site add constraint fk_site_site_address_id foreign key (site_address_id) references site_address (id);

create index ix_source_base_target_id on source_base (target_id);
alter table source_base add constraint fk_source_base_target_id foreign key (target_id) references target_base (id);

create index ix_stockforecast_inner_report_id on stockforecast (inner_report_id);
alter table stockforecast add constraint fk_stockforecast_inner_report_id foreign key (inner_report_id) references inner_report (id);

create index ix_sub_section_section_id on sub_section (section_id);
alter table sub_section add constraint fk_sub_section_section_id foreign key (section_id) references section (id);

create index ix_tevent_many_event_id on tevent_many (event_id);
alter table tevent_many add constraint fk_tevent_many_event_id foreign key (event_id) references tevent_one (id);

alter table tevent_one add constraint fk_tevent_one_event_id foreign key (event_id) references tevent (id);

create index ix_t_detail_with_other_namexxxyy_master_id on t_detail_with_other_namexxxyy (master_id);
alter table t_detail_with_other_namexxxyy add constraint fk_t_detail_with_other_namexxxyy_master_id foreign key (master_id) references t_atable_thatisrelatively (id);

create index ix_ttruck_holder_truck_plate_no on ttruck_holder (truck_plate_no);
alter table ttruck_holder add constraint fk_ttruck_holder_truck_plate_no foreign key (truck_plate_no) references tcar (plate_no);

create index ix_ttruck_holder_basic_id on ttruck_holder (basic_id);
alter table ttruck_holder add constraint fk_ttruck_holder_basic_id foreign key (basic_id) references e_basic (id);

create index ix_ttruck_holder_item_owner_id on ttruck_holder_item (owner_id);
alter table ttruck_holder_item add constraint fk_ttruck_holder_item_owner_id foreign key (owner_id) references ttruck_holder (id);

create index ix_twheel_owner_plate_no on twheel (owner_plate_no);
alter table twheel add constraint fk_twheel_owner_plate_no foreign key (owner_plate_no) references tcar (plate_no);

alter table tire add constraint fk_tire_wheel foreign key (wheel) references wheel (id);

create index ix_tree_entity_parent_id on tree_entity (parent_id);
alter table tree_entity add constraint fk_tree_entity_parent_id foreign key (parent_id) references tree_entity (id);

create index ix_trip_vehicle_driver_id on trip (vehicle_driver_id);
alter table trip add constraint fk_trip_vehicle_driver_id foreign key (vehicle_driver_id) references vehicle_driver (id);

create index ix_trip_address_id on trip (address_id);
alter table trip add constraint fk_trip_address_id foreign key (address_id) references o_address (id);

create index ix_type_sub_type_id on [type] (sub_type_id);
alter table [type] add constraint fk_type_sub_type_id foreign key (sub_type_id) references sub_type (sub_type_id);

create index ix_usib_child_parent_id on usib_child (parent_id);
alter table usib_child add constraint fk_usib_child_parent_id foreign key (parent_id) references usib_parent (id);

alter table usib_child_sibling add constraint fk_usib_child_sibling_child_id foreign key (child_id) references usib_child (id);

create index ix_ut_detail_utmaster_id on ut_detail (utmaster_id);
alter table ut_detail add constraint fk_ut_detail_utmaster_id foreign key (utmaster_id) references ut_master (id);

create index ix_uutwo_master_id on uutwo (master_id);
alter table uutwo add constraint fk_uutwo_master_id foreign key (master_id) references uuone (id);

alter table oto_user add constraint fk_oto_user_account_id foreign key (account_id) references oto_account (id);

create index ix_c_user_group_id on c_user (group_id);
alter table c_user add constraint fk_c_user_group_id foreign key (group_id) references c_group (id);

create index ix_em_user_role_user_id on em_user_role (user_id);
alter table em_user_role add constraint fk_em_user_role_user_id foreign key (user_id) references em_user (id);

create index ix_em_user_role_role_id on em_user_role (role_id);
alter table em_user_role add constraint fk_em_user_role_role_id foreign key (role_id) references em_role (id);

create index ix_vehicle_lease_id on vehicle (lease_id);
alter table vehicle add constraint fk_vehicle_lease_id foreign key (lease_id) references vehicle_lease (id);

create index ix_vehicle_car_ref_id on vehicle (car_ref_id);
alter table vehicle add constraint fk_vehicle_car_ref_id foreign key (car_ref_id) references truck_ref (id);

create index ix_vehicle_truck_ref_id on vehicle (truck_ref_id);
alter table vehicle add constraint fk_vehicle_truck_ref_id foreign key (truck_ref_id) references truck_ref (id);

create index ix_vehicle_driver_vehicle_id on vehicle_driver (vehicle_id);
alter table vehicle_driver add constraint fk_vehicle_driver_vehicle_id foreign key (vehicle_id) references vehicle (id);

create index ix_vehicle_driver_address_id on vehicle_driver (address_id);
alter table vehicle_driver add constraint fk_vehicle_driver_address_id foreign key (address_id) references o_address (id);

create index ix_version_child_parent_id on version_child (parent_id);
alter table version_child add constraint fk_version_child_parent_id foreign key (parent_id) references version_parent (id);

create index ix_version_toy_child_id on version_toy (child_id);
alter table version_toy add constraint fk_version_toy_child_id foreign key (child_id) references version_child (id);

create index ix_warehouses_officezoneid on warehouses (officezoneid);
alter table warehouses add constraint fk_warehouses_officezoneid foreign key (officezoneid) references zones (id);

create index ix_warehousesshippingzones_warehouses on warehousesshippingzones (warehouseid);
alter table warehousesshippingzones add constraint fk_warehousesshippingzones_warehouses foreign key (warehouseid) references warehouses (id);

create index ix_warehousesshippingzones_zones on warehousesshippingzones (shippingzoneid);
alter table warehousesshippingzones add constraint fk_warehousesshippingzones_zones foreign key (shippingzoneid) references zones (id);

create index ix_sa_wheel_tire on sa_wheel (tire);
alter table sa_wheel add constraint fk_sa_wheel_tire foreign key (tire) references sa_tire (id);

create index ix_sa_wheel_car on sa_wheel (car);
alter table sa_wheel add constraint fk_sa_wheel_car foreign key (car) references sa_car (id);

create index ix_g_who_props_otm_who_created_id on g_who_props_otm (who_created_id);
alter table g_who_props_otm add constraint fk_g_who_props_otm_who_created_id foreign key (who_created_id) references g_user (id);

create index ix_g_who_props_otm_who_modified_id on g_who_props_otm (who_modified_id);
alter table g_who_props_otm add constraint fk_g_who_props_otm_who_modified_id foreign key (who_modified_id) references g_user (id);

create index ix_with_zero_parent_id on with_zero (parent_id);
alter table with_zero add constraint fk_with_zero_parent_id foreign key (parent_id) references parent (id);

alter table hembi_bean
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hembi_bean set (system_versioning = on (history_table=dbo.hembi_bean_history));
alter table hx_link
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hx_link set (system_versioning = on (history_table=dbo.hx_link_history));
alter table hi_link
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hi_link set (system_versioning = on (history_table=dbo.hi_link_history));
alter table hi_link_doc
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hi_link_doc set (system_versioning = on (history_table=dbo.hi_link_doc_history));
alter table hi_tone
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hi_tone set (system_versioning = on (history_table=dbo.hi_tone_history));
alter table hi_tthree
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hi_tthree set (system_versioning = on (history_table=dbo.hi_tthree_history));
alter table hi_ttwo
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hi_ttwo set (system_versioning = on (history_table=dbo.hi_ttwo_history));
alter table hsd_setting
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hsd_setting set (system_versioning = on (history_table=dbo.hsd_setting_history));
alter table hsd_user
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table hsd_user set (system_versioning = on (history_table=dbo.hsd_user_history));
alter table link
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table link set (system_versioning = on (history_table=dbo.link_history));
alter table c_user
    add sys_periodFrom datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT SYSUTCDATETIME(),
        sys_periodTo   datetime2 GENERATED ALWAYS AS ROW END   NOT NULL DEFAULT '9999-12-31T23:59:59.9999999',
period for system_time (sys_periodFrom, sys_periodTo);
alter table c_user set (system_versioning = on (history_table=dbo.c_user_history));
